@@ -151,7 +151,7 @@ static int csch_attrib_set_nofree(csch_attribs_t *attribs, int prio, const char
 	if (strdup_val)
 		a->val = ((val == NULL) ? NULL : rnd_strdup(val));
 	else
-		a->val = val;
+		a->val = (char *)val; /* caller explicitly asked for this cast */
 	append_src(a, prio, source, 0);
 	if (attr_out != NULL) *attr_out = a;
 	return 0;
@@ -211,7 +211,8 @@ int csch_attrib_seti(csch_attribs_t *attribs, int prio, const char *key, long id
 	return -1;
 }
 
-static int csch_attrib_set_arr_nofree(csch_attribs_t *attribs, int prio, const char *key, const vts0_t *val, csch_source_arg_t *source, csch_attrib_t **attr_out)
+/* how is: 0=overwrite whole arr; -1=prepend; +1=append */
+static int csch_attrib_set_arr_nofree(csch_attribs_t *attribs, int prio, const char *key, const vts0_t *val, csch_source_arg_t *source, csch_attrib_t **attr_out, int how)
 {
 	long n;
 	csch_attrib_t *a = htsp_get(attribs, key);
@@ -229,25 +230,68 @@ static int csch_attrib_set_arr_nofree(csch_attribs_t *attribs, int prio, const c
 			if (attr_out != NULL) *attr_out = a;
 			return -1;
 		}
-		attr_free_val(a);
+		if (how == 0)
+			attr_free_val(a);
 	}
 	append_src(a, prio, source, 0);
-	for(n = 0; n < a->arr.used; n++)
-		free(a->arr.array[n]);
-	a->arr.used = 0;
-	for(n = 0; n < val->used; n++)
-		vts0_set(&a->arr, n, rnd_strdup(val->array[n]));
+
+	if (how == 0) {
+		/* overwrite */
+		for(n = 0; n < a->arr.used; n++)
+			free(a->arr.array[n]);
+		a->arr.used = 0;
+		for(n = 0; n < val->used; n++)
+			vts0_set(&a->arr, n, rnd_strdup(val->array[n]));
+	}
+	else if (how > 1) {
+		/* append */
+		for(n = 0; n < val->used; n++)
+			vts0_append(&a->arr, rnd_strdup(val->array[n]));
+	}
+	else {
+		/* prepend */
+		for(n = 0; n < val->used; n++)
+			vts0_append(&a->arr, rnd_strdup(val->array[n]));
+	}
+
 	if (attr_out != NULL) *attr_out = a;
 	return 0;
 }
 
 int csch_attrib_set_arr(csch_attribs_t *attribs, int prio, const char *key, const vts0_t *val, csch_source_arg_t *source, csch_attrib_t **attr_out)
 {
-	int res = csch_attrib_set_arr_nofree(attribs, prio, key, val, source, attr_out);
+	int res = csch_attrib_set_arr_nofree(attribs, prio, key, val, source, attr_out, 0);
 	csch_attr_src_free(source);
 	return res;
 }
 
+int csch_attrib_append(csch_attribs_t *attribs, int prio, const char *key, const char *val, csch_source_arg_t *source)
+{
+	int res;
+	vts0_t tmp = {0};
+
+	tmp.used = tmp.alloced = 1;
+	tmp.array = (char **)&val;
+
+	res = csch_attrib_set_arr_nofree(attribs, prio, key, &tmp, source, NULL, 1);
+	csch_attr_src_free(source);
+	return res;
+}
+
+int csch_attrib_prepend(csch_attribs_t *attribs, int prio, const char *key, const char *val, csch_source_arg_t *source)
+{
+	int res;
+	vts0_t tmp = {0};
+
+	tmp.used = tmp.alloced = 1;
+	tmp.array = (char **)&val;
+
+	res = csch_attrib_set_arr_nofree(attribs, prio, key, &tmp, source, NULL, -1);
+	csch_attr_src_free(source);
+	return res;
+}
+
+
 int csch_attrib_set_arr_c(csch_attribs_t *attribs, int prio, const char *key, const char **val, csch_source_arg_t *source, csch_attrib_t **attr_out)
 {
 	int res;
@@ -259,7 +303,7 @@ int csch_attrib_set_arr_c(csch_attribs_t *attribs, int prio, const char *key, co
 	tmp.alloced = tmp.used;
 	tmp.array = (char **)val;
 
-	res = csch_attrib_set_arr_nofree(attribs, prio, key, &tmp, source, attr_out);
+	res = csch_attrib_set_arr_nofree(attribs, prio, key, &tmp, source, attr_out, 0);
 	csch_attr_src_free(source);
 	return res;
 }
@@ -318,17 +362,52 @@ void csch_attrib_copy_all(csch_attribs_t *dst, const csch_attribs_t *src)
 	}
 }
 
-int csch_attrib_apply(csch_attribs_t *dst, const csch_attribs_t *src, csch_source_arg_t *source)
+void csch_attrib_copy_all_undoable(csch_sheet_t *sheet, csch_cgrp_t *dst_obj, const csch_attribs_t *src)
+{
+	htsp_entry_t *e;
+	csch_attribs_t *dst = &dst_obj->attr;
+
+	for(e = htsp_first(src); e; e = htsp_next(src, e)) {
+		csch_attrib_t *s = csch_attrib_dup(e->value);
+		csch_attrib_t *d = csch_attrib_get(dst, e->key);
+		if (s->val == NULL) {
+TODO("figure how to merge arrays");
+			rnd_message(RND_MSG_ERROR, "csch_attrib_copy_all_undoable(): internal error: can't copy/merge arrya type attribte '%s'\n", e->key);
+		}
+		else {
+TODO("handle if src is an array");
+			if ((d == NULL) || ((d->val != NULL) && (strcmp(d->val, s->val) != 0))) {
+				char *sep, *ssrc = s->source.array[0];
+				long len;
+				csch_source_arg_t *src;
+
+				sep = strchr(ssrc, ':');
+				sep++;
+				len = strlen(sep);
+
+				src = malloc(sizeof(csch_source_arg_t) + len + 1);
+				src->type[0] = 'c';
+				src->type[1] = '\0';
+				memcpy(src->src_desc, sep, len+1);
+
+				csch_attr_modify_str(sheet, dst_obj, s->prio, e->key, s->val, src, 1);
+			}
+		}
+	}
+}
+
+int csch_attrib_apply(csch_attribs_t *dst, const csch_attribs_t *src, csch_source_arg_t *source, int *force_prio)
 {
 	htsp_entry_t *e;
 	int err = 0;
 
 	for(e = htsp_first(src); e; e = htsp_next(src, e)) {
 		csch_attrib_t *a = e->value;
+		int prio = (force_prio != NULL) ? *force_prio : a->prio;
 		if (a->val != NULL)
-			err |= csch_attrib_set_nofree(dst, a->prio, a->key, a->val, source, NULL, 1);
+			err |= csch_attrib_set_nofree(dst, prio, a->key, a->val, source, NULL, 1);
 		else
-			err |= csch_attrib_set_arr_nofree(dst, a->prio, a->key, &a->arr, source, NULL);
+			err |= csch_attrib_set_arr_nofree(dst, prio, a->key, &a->arr, source, NULL, 0);
 	}
 
 	csch_attr_src_free(source);
@@ -458,7 +537,7 @@ static int undo_attr_modify_swap(void *udata)
 	if (u->a != NULL)
 		htsp_set(&u->obj->attr, u->a->key, u->a);
 	else
-		htsp_popentry(&u->obj->attr, a->key);
+		htsp_popentry(&u->obj->attr, u->key);
 	u->a = a;
 
 	csch_attr_side_effects(u->obj, u->key);
@@ -526,6 +605,12 @@ void csch_attr_modify_str(csch_sheet_t *sheet, csch_cgrp_t *obj, int prio, const
 void csch_attr_modify_del(csch_sheet_t *sheet, csch_cgrp_t *obj, const char *key, int undoable)
 {
 	undo_attr_modify_t utmp, *u = &utmp;
+	csch_attrib_t *a;
+
+	/* skip creating an undo slot for a non-existing attribute's del (no-op) */
+	a = htsp_get(&obj->attr, key);
+	if (a == NULL)
+		return;
 
 	if (undoable) u = uundo_append(&sheet->undo, &undo_attr_modify, sizeof(undo_attr_modify_t));
 
@@ -537,6 +622,24 @@ void csch_attr_modify_del(csch_sheet_t *sheet, csch_cgrp_t *obj, const char *key
 	if (undoable) csch_undo_inc_serial(sheet);
 }
 
+void csch_attr_modify_rename(csch_sheet_t *sheet, csch_cgrp_t *grp, csch_attrib_t *a, const char *new_name, csch_source_arg_t *src, int undoable)
+{
+	int prio = a->prio;
+
+	assert(strcmp(new_name, a->key) != 0);
+
+	if (undoable)
+		uundo_freeze_serial(&sheet->undo);
+
+	csch_attr_modify_str(sheet, grp, prio, new_name, a->val, src, undoable);
+	csch_attr_modify_del(sheet, grp, a->key, undoable);
+
+	if (undoable) {
+		uundo_unfreeze_serial(&sheet->undo);
+		uundo_inc_serial(&sheet->undo);
+	}
+}
+
 
 static const char *NEWVAL_DEL = "NEWVAL_DEL";
 typedef struct {
