@@ -2,9 +2,9 @@
  *                            COPYRIGHT
  *
  *  cschem - modular/flexible schematics editor - libcschem (core library)
- *  Copyright (C) 2018,2022 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2018,2022,2023 Tibor 'Igor2' Palinkas
  *
- *  (Supported by NLnet NGI0 PET Fund in 2022)
+ *  (Supported by NLnet NGI0 PET Fund in 2022, Entrust in 2023)
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -231,8 +231,12 @@ csch_inline void conn_recalc_add_line(csch_conn_t *conn, csch_chdr_t *o1, csch_c
 
 void csch_conn_recalc_coords(csch_conn_t *conn)
 {
-	int i1, i2;
+	int i1, i2, readd_rtree;
 	vtl0_t tmp = {0};
+	csch_sheet_t *sheet = conn->hdr.sheet;
+
+	/* try removing the object from its rtree; if it succeeds, add it back after the recalculation */
+	readd_rtree = (sheet != NULL) && (csch_cobj_rtree_del(sheet, &conn->hdr) == 0);
 
 	csch_bbox_reset(&conn->hdr.bbox);
 	conn->coords.used = 0;
@@ -240,6 +244,9 @@ void csch_conn_recalc_coords(csch_conn_t *conn)
 		for(i2 = i1+1; i2 < conn->conn.used; i2++)
 			conn_recalc_add_line(conn, conn->conn.array[i1], conn->conn.array[i2], &tmp);
 
+	if (readd_rtree)
+		csch_cobj_rtree_add(sheet, &conn->hdr);
+
 	vtl0_uninit(&tmp);
 }
 
@@ -253,6 +260,8 @@ void csch_conn_update(csch_sheet_t *sheet, csch_conn_t *conn, int do_xform)
 	csch_cobj_rtree_del(sheet, &conn->hdr);
 	csch_conn_recalc_coords(conn);
 	csch_cobj_rtree_add(sheet, &conn->hdr);
+
+	csch_cobj_bbox_changed(sheet, &conn->hdr);
 }
 
 static long find_obj_in_conn(const csch_chdr_t *obj, const csch_conn_t *conn)
@@ -320,7 +329,7 @@ typedef struct {
 	unsigned add:1; /* if 1, swap should add, else swap should del (swap always inverts afterwards) */
 } undo_conn_obj_t;
 
-static int undo_conn_obj_swap(void *udata)
+static int undo_conn_obj_swap_(void *udata, int update)
 {
 	undo_conn_obj_t *u = udata;
 
@@ -329,11 +338,19 @@ static int undo_conn_obj_swap(void *udata)
 	else
 		csch_conn_del_obj_(u->sheet, u->conn, u->obj);
 
+	if (update) /* recalc coords (the graphics) when called from undo/redo */
+		csch_conn_update(u->sheet, u->conn, 0);
+
 	u->add = !u->add;
 
 	return 0;
 }
 
+static int undo_conn_obj_swap(void *udata)
+{
+	return undo_conn_obj_swap_(udata, 1);
+}
+
 static void undo_conn_obj_print(void *udata, char *dst, size_t dst_len)
 {
 	undo_conn_obj_t *u = udata;
@@ -378,7 +395,7 @@ csch_inline int csch_conn_do_obj(csch_sheet_t *sheet, csch_conn_t *conn, csch_ch
 	u->obj = obj;
 	u->add = add;
 
-	res = undo_conn_obj_swap(u);
+	res = undo_conn_obj_swap_(u, 0); /* update 0: expect the caller to do the update */
 	if (undoable) csch_undo_inc_serial(sheet);
 	if (cidx != NULL)
 		*cidx = u->last_cidx;
@@ -407,7 +424,7 @@ long csch_conn_add_obji(csch_sheet_t *sheet, csch_conn_t *conn, csch_chdr_t *obj
 /* Return conn where:
    obj1 is in the connection and another object from obj2's parent is in too
    (or return NULL if not found) */
-csch_inline csch_conn_t *conn_find_common(csch_chdr_t *obj1, csch_chdr_t *obj2, long *o1i)
+csch_inline csch_conn_t *conn_find_common(csch_chdr_t *obj1, csch_chdr_t *obj2, long *o1i, csch_conn_t *ignore)
 {
 	long n, m;
 	csch_conn_t *conn;
@@ -415,6 +432,8 @@ csch_inline csch_conn_t *conn_find_common(csch_chdr_t *obj1, csch_chdr_t *obj2,
 
 	for(n = 0; n < obj1->conn.used; n++) {
 		conn = obj1->conn.array[n];
+		if (conn == ignore)
+			continue;
 		for(m = 0; m < conn->conn.used; m++) {
 			o = conn->conn.array[m];
 			if (o->parent == obj2->parent) {
@@ -430,13 +449,26 @@ csch_inline csch_conn_t *conn_find_common(csch_chdr_t *obj1, csch_chdr_t *obj2,
 	return NULL;
 }
 
+csch_conn_t *csch_conn_find_mergable(csch_sheet_t *sheet, csch_conn_t *conn, csch_chdr_t *owner)
+{
+	long n;
+	
+	for(n = 0; n < conn->conn.used; n++) {
+		csch_chdr_t *co = conn->conn.array[n];
+		csch_conn_t *res = conn_find_common(co, owner, NULL, conn);
+		if (res != NULL)
+			return res;
+	}
+	return NULL;
+}
+
 csch_inline csch_conn_t *csch_conn_auto_add_(csch_sheet_t *sheet, csch_chdr_t *obj1, csch_chdr_t *obj2, long *obj1i)
 {
 	csch_conn_t *conn;
 
 	/* check if there's an existing connection we can extend; requirement is:
 	   obj1/2 is in the connection and another object from obj2/1's parent is in too */
-	conn = conn_find_common(obj1, obj2, obj1i);
+	conn = conn_find_common(obj1, obj2, obj1i, NULL);
 	if (conn != NULL) {
 		if (find_obj_in_conn(obj2, conn) < 0) {
 			csch_conn_add_obj(sheet, conn, obj2, 1);
@@ -445,7 +477,7 @@ csch_inline csch_conn_t *csch_conn_auto_add_(csch_sheet_t *sheet, csch_chdr_t *o
 		return conn;
 	}
 
-	conn = conn_find_common(obj2, obj1, NULL);
+	conn = conn_find_common(obj2, obj1, NULL, NULL);
 	if (conn != NULL) {
 		*obj1i = find_obj_in_conn(obj1, conn);
 		if (*obj1i < 0) {
@@ -616,7 +648,7 @@ static void csch_conn_remove_disconnected_objs(csch_sheet_t *sheet, csch_conn_t
 	}
 }
 
-void csch_recalc_obj_conn(csch_sheet_t *sheet, csch_chdr_t *wobj, csch_displayer_t target)
+void csch_recalc_obj_conn(csch_sheet_t *sheet, csch_chdr_t *wobj, csch_displayer_t target, csch_displayer_t target2)
 {
 	find_conn_t ctx = {0};
 	long n;
@@ -632,8 +664,11 @@ void csch_recalc_obj_conn(csch_sheet_t *sheet, csch_chdr_t *wobj, csch_displayer
 	}
 
 	/* we may be called on deleted objects to get their conns removed */
-	if (!csch_obj_is_deleted(wobj))
+	if (!csch_obj_is_deleted(wobj)) {
 		csch_rtree_search_obj(&sheet->dsply[target], &wobj->bbox, find_conn_cb, &ctx);
+		if (target2 != CSCH_DSPLY_invalid)
+			csch_rtree_search_obj(&sheet->dsply[target2], &wobj->bbox, find_conn_cb, &ctx);
+	}
 
 	for(n = ctx.keep.used-1; n >= 0; n--) {
 		if (ctx.keep.array[n]) {
@@ -664,7 +699,7 @@ static void csch_conn_auto_recalc_(csch_sheet_t *sheet, csch_chdr_t *obj, int in
 			if (is_deleted)
 				csch_conn_auto_del_obj_all(sheet, obj);
 			else
-				csch_recalc_obj_conn(sheet, obj, in_term ? CSCH_DSPLY_WIRE : CSCH_DSPLY_HUBTERM);
+				csch_recalc_obj_conn(sheet, obj, in_term ? CSCH_DSPLY_WIRE : CSCH_DSPLY_HUBTERM, in_term ? CSCH_DSPLY_HUBTERM : CSCH_DSPLY_invalid);
 		}
 	}
 }
@@ -675,3 +710,22 @@ void csch_conn_auto_recalc(csch_sheet_t *sheet, csch_chdr_t *obj)
 }
 
 
+void csch_conn_auto_recalc_geo(csch_sheet_t *sheet, csch_chdr_t *obj)
+{
+	long n;
+	for(n = 0; n < obj->conn.used; n++)
+		csch_conn_recalc_coords((csch_conn_t *)obj->conn.array[n]);
+
+	/* recurse into children */
+	if (csch_obj_is_grp(obj)) {
+		csch_cgrp_t *grp = (csch_cgrp_t *)obj;
+		htip_entry_t *e;
+		for(e = htip_first(&grp->id2obj); e != NULL; e = htip_next(&grp->id2obj, e))
+			csch_conn_auto_recalc_geo(sheet, e->value);
+	}
+	else if (obj->type == CSCH_CTYPE_CONN)
+		csch_conn_recalc_coords((csch_conn_t *)obj);
+}
+
+
+
