@@ -2,9 +2,9 @@
  *                            COPYRIGHT
  *
  *  cschem - modular/flexible schematics editor - libcschem (core library)
- *  Copyright (C) 2018..2020,2022 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2018..2020,2022,2023 Tibor 'Igor2' Palinkas
  *
- *  (Supported by NLnet NGI0 PET Fund in 2022)
+ *  (Supported by NLnet NGI0 PET Fund in 2022, Entrust in 2023)
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -29,6 +29,8 @@
 
 #include "config.h"
 
+#include <stdarg.h>
+
 #include "concrete.h"
 
 #include <librnd/core/misc_util.h>
@@ -42,6 +44,7 @@
 #include "event.h"
 #include "cnc_grp.h"
 #include "cnc_grp_child.h"
+#include "cnc_conn.h"
 #include "cnc_obj.h"
 #include "cnc_any_obj.h"
 #include "cnc_loop.h"
@@ -276,6 +279,8 @@ void csch_cgrp_update(csch_sheet_t *sheet, csch_cgrp_t *grp, int do_xform)
 	csch_cobj_rtree_del(sheet, &grp->hdr);
 	csch_cgrp_bbox_update(sheet, grp);
 	csch_cobj_rtree_add(sheet, &grp->hdr);
+
+	csch_cobj_bbox_changed(sheet, &grp->hdr);
 }
 
 void csch_cgrp_render(csch_sheet_t *sheet, csch_cgrp_t *grp)
@@ -305,7 +310,7 @@ void csch_cgrp_attrib_update(csch_sheet_t *sheet, csch_cgrp_t *grp, int prio, co
 
 csch_ahdr_t *csch_cgrp_get_abstract(csch_sheet_t *sheet, const csch_cgrp_t *grp, long idx)
 {
-	csch_project_t *prj = sheet->parent;
+	csch_project_t *prj = (csch_project_t *)sheet->hidlib.project;
 
 	if ((prj == NULL) || (prj->abst == NULL))
 		return NULL;
@@ -612,6 +617,7 @@ static int undo_cgrp_modify_swap(void *udata)
 	rnd_swap(double, u->spec_rot, u->grp->spec_rot);
 
 	csch_cgrp_update(u->grp->hdr.sheet, u->grp, 1);
+	csch_conn_auto_recalc_geo(u->grp->hdr.sheet, &u->grp->hdr);
 	csch_sheet_set_changed(u->grp->hdr.sheet, 1);
 
 	csch_cobj_redraw(u->grp);
@@ -701,3 +707,57 @@ void csch_cgrp_modify_uuid(csch_sheet_t *sheet, csch_cgrp_t *grp, minuid_bin_t *
 	if (undoable) csch_undo_inc_serial(sheet);
 }
 
+
+/*** transformations ***/
+void csch_cgrp_inverse_xform(csch_cgrp_t *parent, csch_coord_t *x, csch_coord_t *y, long num_crds)
+{
+	g2d_xform_t mx;
+
+	csch_cgrp_inverse_matrix(&mx, parent);
+
+	for(; num_crds > 0; num_crds--,x++,y++) {
+		g2d_vect_t v1;
+		v1.x = *x; v1.y = *y;
+		v1 = g2d_xform_vect2vect(mx, v1);
+		*x = v1.x; *y = v1.y;
+	}
+}
+
+void csch_cgrp_vinverse_xform(csch_cgrp_t *parent, ...)
+{
+	g2d_xform_t mx;
+	va_list ap;
+
+	csch_cgrp_inverse_matrix(&mx, parent);
+	va_start(ap, parent);
+
+	for(;;) {
+		g2d_vect_t v1;
+		csch_coord_t *x, *y;
+
+		x = va_arg(ap, csch_coord_t *); if (x == NULL) break;
+		y = va_arg(ap, csch_coord_t *); if (y == NULL) break;
+
+		v1.x = *x; v1.y = *y;
+		v1 = g2d_xform_vect2vect(mx, v1);
+		*x = v1.x; *y = v1.y;
+	}
+
+	va_end(ap);
+}
+
+/* based on https://math.stackexchange.com/questions/13150/extracting-rotation-scale-values-from-2d-transformation-matrix/13165#13165 */
+double csch_cgrp_extract_rot(g2d_xform_t mx)
+{
+	return atan2(mx.v[3], mx.v[4]) * RND_RAD_TO_DEG;
+}
+
+void csch_cgrp_extract_scale(g2d_xform_t mx, double *sx, double *sy)
+{
+	*sx = sqrt(mx.v[0] * mx.v[0] + mx.v[1] * mx.v[1]);
+	*sy = sqrt(mx.v[3] * mx.v[3] + mx.v[4] * mx.v[4]);
+
+	/* This is needed for mirroring */
+	if (mx.v[0] < 0) *sx = -(*sx);
+	if (mx.v[4] < 0) *sy = -(*sy);
+}
