@@ -85,23 +85,47 @@ csch_text_t *csch_text_alloc(csch_sheet_t *sheet, csch_cgrp_t *parent, csch_oid_
 	return text;
 }
 
-csch_text_t *csch_text_dup(csch_sheet_t *sheet, csch_cgrp_t *parent, const csch_text_t *src, int keep_id)
+csch_text_t *csch_text_dup(csch_sheet_t *sheet, csch_cgrp_t *parent, const csch_text_t *src, int keep_id, int inst2spec)
 {
 	csch_text_t *dst = csch_text_alloc(sheet, parent, CSCH_KEEP_OID(parent, src->hdr.oid));
-	dst->spec1 = src->spec1;
-	dst->spec2 = src->spec2;
-	dst->spec_rot = src->spec_rot;
-	dst->spec_mirx = src->spec_mirx;
-	dst->spec_miry = src->spec_miry;
-	dst->inst_rot = src->inst_rot;
-	dst->inst_raw_rot = src->inst_raw_rot;
-	dst->inst_mirx = src->inst_mirx;
-	dst->inst_miry = src->inst_miry;
+
 	dst->has_bbox = src->has_bbox;
 	dst->halign = src->halign;
 	dst->text = rnd_strdup(src->text);
 	dst->dyntext = src->dyntext;
 
+
+
+	if (inst2spec) {
+		g2d_xform_t mx;
+		g2d_vect_t v1 = src->inst1, v2 = src->inst2;
+
+		csch_cgrp_inverse_matrix(&mx, parent);
+
+		dst->spec_rot = src->inst_rot - parent->xform.rot;
+		dst->spec_mirx = src->inst_mirx ^ parent->xform.mirx;
+		dst->spec_miry = src->inst_miry ^ parent->xform.miry;
+
+		v1 = g2d_xform_vect2vect(mx, v1);
+		v2 = g2d_xform_vect2vect(mx, v2);
+		dst->spec1 = v1;
+		dst->spec2 = v2;
+		csch_text_update(sheet, dst, 1);
+	}
+	else {
+		dst->spec1 = src->spec1;
+		dst->spec2 = src->spec2;
+
+		dst->spec_rot = src->spec_rot;
+		dst->spec_mirx = src->spec_mirx;
+		dst->spec_miry = src->spec_miry;
+		dst->inst_rot = src->inst_rot;
+		dst->inst_raw_rot = src->inst_raw_rot;
+		dst->inst_mirx = src->inst_mirx;
+		dst->inst_miry = src->inst_miry;
+	}
+
+
 	csch_chdr_copy_meta4dup(&dst->hdr, &src->hdr);
 	return dst;
 }
@@ -136,14 +160,16 @@ void csch_text_invalidate_all_grp(csch_cgrp_t *grp, int dynonly)
 
 void csch_text_invalidate_all_sheet(csch_sheet_t *sheet, int dynonly)
 {
+	csch_cobj_redraw_freeze(sheet);
 	csch_text_invalidate_all_grp(&sheet->direct, dynonly);
+	csch_cobj_redraw_unfreeze(sheet);
 }
 
 void csch_text_invalidate_all_project(csch_project_t *prj, int dynonly)
 {
 	long n;
-	for(n = 0; n < prj->sheets.used; n++)
-		csch_text_invalidate_all_sheet(prj->sheets.array[n], dynonly);
+	for(n = 0; n < prj->hdr.designs.used; n++)
+		csch_text_invalidate_all_sheet(prj->hdr.designs.array[n], dynonly);
 }
 
 
@@ -224,8 +250,11 @@ void csch_text_update_bbox(csch_sheet_t *sheet, csch_text_t *text)
 
 void csch_text_update(csch_sheet_t *sheet, csch_text_t *text, int do_xform)
 {
+	if (!text->bbox_calcing)
+		csch_cobj_rtree_del(sheet, &text->hdr);
+
 	if (!text->bbox_calcing) {
-		csch_cobj_redraw_freeze();
+		csch_cobj_redraw_freeze(sheet);
 		csch_cobj_update_pen(&text->hdr);
 
 		if (!text->has_bbox && !text->bbox_calced) {
@@ -235,7 +264,7 @@ void csch_text_update(csch_sheet_t *sheet, csch_text_t *text, int do_xform)
 			text->bbox_calced = 1;
 			text->bbox_calcing = 0;
 		}
-		csch_cobj_redraw_unfreeze();
+		csch_cobj_redraw_unfreeze(sheet);
 	}
 
 	text->spec15.x = text->spec2.x; text->spec15.y = text->spec1.y;
@@ -274,13 +303,13 @@ void csch_text_update(csch_sheet_t *sheet, csch_text_t *text, int do_xform)
 	if (do_xform)
 		csch_text_update_xform(sheet, text);
 
-	if (!text->bbox_calcing)
-		csch_cobj_rtree_del(sheet, &text->hdr);
 
 	csch_text_update_bbox(sheet, text);
 
-	if (!text->bbox_calcing)
+	if (!text->bbox_calcing) {
 		csch_cobj_rtree_add(sheet, &text->hdr);
+		csch_cobj_bbox_changed(sheet, &text->hdr);
+	}
 }
 
 int csch_text_invalid_chars(csch_text_t *txt)
@@ -372,7 +401,7 @@ static void text_move(csch_sheet_t *sheet, csch_chdr_t *obj, csch_coord_t dx, cs
 
 static void text_copy(csch_sheet_t *sheet, csch_chdr_t *obj, csch_coord_t dx, csch_coord_t dy, int undoable)
 {
-	csch_text_t *text = csch_text_dup(sheet, obj->parent, (csch_text_t *)obj, 0);
+	csch_text_t *text = csch_text_dup(sheet, obj->parent, (csch_text_t *)obj, 0, 0);
 	if (text != NULL) {
 		text->spec1.x += dx; text->spec1.y += dy;
 		text->spec2.x += dx; text->spec2.y += dy;
@@ -384,11 +413,15 @@ static void text_copy(csch_sheet_t *sheet, csch_chdr_t *obj, csch_coord_t dx, cs
 static void text_rotate(csch_sheet_t *sheet, csch_chdr_t *obj, csch_coord_t rcx, csch_coord_t rcy, double da, int undoable)
 {
 	csch_text_t *text = (csch_text_t *)obj;
-	double rot = fmod(text->spec_rot - da, 360.0);
-	double rad = -rot / RND_RAD_TO_DEG;
+	double rad, rot = fmod(text->spec_rot - da, 360.0);
 	csch_coord_t sx = text->spec2.x - text->spec1.x, sy = text->spec2.y - text->spec1.y;
 	csch_coord_t x1 = text->spec1.x, y1 = text->spec1.y, x2, y2;
 
+	if (text->spec_mirx ^ text->spec_miry)
+		rot = -rot;
+
+	rad = -rot / RND_RAD_TO_DEG;
+
 	csch_rotate_pt(&x1, &y1, rcx, rcy, cos(rad), sin(rad));
 	x2 = x1 + sx; y2 = y1 + sy;
 
@@ -402,6 +435,9 @@ static void text_rotate90(csch_sheet_t *sheet, csch_chdr_t *obj, csch_coord_t rc
 	csch_coord_t sx = text->spec2.x - text->spec1.x, sy = text->spec2.y - text->spec1.y;
 	csch_coord_t x1 = text->spec1.x, y1 = text->spec1.y, x2, y2;
 
+	if (text->spec_mirx ^ text->spec_miry)
+		n = -n;
+
 	csch_rotate90_pt(&x1, &y1, rcx, rcy, n);
 	x2 = x1 + sx; y2 = y1 + sy;
 	csch_text_modify(sheet, text, &x1, &y1, &x2, &y2, &rot, NULL, NULL, NULL, NULL, NULL, NULL, undoable, 0);
