@@ -34,6 +34,7 @@
 #include "event.h"
 #include "cnc_obj.h"
 #include "cnc_grp.h"
+#include "project.h"
 #include "plug_library.h"
 
 static const char *dsply_names[CSCH_DSPLY_max] = {
@@ -76,7 +77,9 @@ csch_sheet_t *csch_sheet_init(csch_sheet_t *sheet, csch_project_t *parent)
 	if (sheet == NULL)
 		return NULL;
 
-	sheet->parent = parent;
+	if (parent != NULL)
+		rnd_project_append_design(&parent->hdr, &sheet->hidlib);
+
 	sheet->uid = next_uid++;
 	htsp_init(&sheet->comm_str, strhash, strkeyeq);
 
@@ -127,8 +130,8 @@ void csch_sheet_uninit(csch_sheet_t *sheet)
 
 	free(sheet->newname);
 	sheet->newname = NULL;
-	free(sheet->loadname);
-	sheet->loadname = NULL;
+	free(sheet->hidlib.loadname);
+	sheet->hidlib.loadname = NULL;
 	free(sheet->loadfmt);
 	sheet->loadfmt = NULL;
 	free(sheet->design_dir);
@@ -139,6 +142,8 @@ void csch_sheet_uninit(csch_sheet_t *sheet)
 void csch_sheet_free(csch_sheet_t *sheet)
 {
 	rnd_event(&sheet->hidlib, CSCH_EVENT_SHEET_PREUNLOAD, NULL);
+	if (sheet->hidlib.project != NULL)
+		rnd_project_remove_design(sheet->hidlib.project, &sheet->hidlib);
 	csch_sheet_uninit(sheet);
 	free(sheet);
 }
@@ -219,7 +224,7 @@ void csch_sheet_set_changed(csch_sheet_t *sheet, int new_val)
 	sheet->changed = new_val;
 
 	if (old != new_val)
-		rnd_event(&sheet->hidlib, RND_EVENT_BOARD_META_CHANGED, NULL);
+		rnd_event(&sheet->hidlib, RND_EVENT_DESIGN_META_CHANGED, NULL);
 
 	if (new_val)
 		rnd_event(&sheet->hidlib, CSCH_EVENT_SHEET_EDITED, NULL);
