@@ -45,6 +45,7 @@
 #include "search.h"
 #include "op_common.h"
 #include "undo.h"
+#include "util_wirenet.h"
 
 extern const csch_ops_t csch_ops_line;
 extern const csch_ops_t csch_ops_arc;
@@ -106,10 +107,8 @@ csch_inline csch_chdr_t *csch_cnc_remove_(csch_sheet_t *sheet, csch_chdr_t *obj,
 	if ((how & CSCH_REM_DEL_EMPTY_PARENT) && (parent != NULL) && (parent->id2obj.used == 0) && (!csch_obj_is_deleted(&parent->hdr)))
 		csch_cnc_remove_(sheet, &parent->hdr, how);
 
-		uundo_unfreeze_serial(&sheet->undo);
-		csch_undo_inc_serial(sheet);
-
-
+	uundo_unfreeze_serial(&sheet->undo);
+	csch_undo_inc_serial(sheet);
 
 	csch_sheet_set_changed(sheet, 1);
 
@@ -175,9 +174,32 @@ TODO("undo problem: dup doesn't preserve oid and that may need to be undone");
 
 	/* restore connections: remove old object and add new object to the same conns */
 	if (restore_conns) {
+
 		for(n = 0; n < conn_save.used; n++) {
-			csch_conn_del_obj(sheet, conn_save.array[n], src, undoable_conns);
-			csch_conn_add_obj(sheet, conn_save.array[n], newo, undoable_conns);
+			csch_conn_t *orig_conn = conn_save.array[n];
+			csch_conn_t *existing = csch_conn_find_mergable(sheet, orig_conn, newo);
+
+			if (existing) {
+				/* corer case: removing the wire from orig_conn may have oprhaned
+				   orig_conn; this call takes care of removing that conn in an
+				   undoable manner */
+				csch_conn_auto_del_obj(sheet, orig_conn, src);
+
+				/* if there is an existing "overlapping" conn in dst group, reuse that
+				   instead of our original conn; this happens when a wire from wirenet A
+				   is moved into wirenet B and the wire creates a new connection to
+				   a terminal that wirenet B already connected. In this case the wire
+				   should be removed from its own connection (orig_conn) and should be
+				   added to the existing connection of wirenet B. */
+				csch_conn_add_obj(sheet, existing, newo, undoable_conns);
+			}
+			else {
+				/* replace object in conn: remove old object (src) and add new
+				   object (newo); this won't change the number of objects in orig_conn,
+				   it can't get orphaned */
+				csch_conn_del_obj(sheet, orig_conn, src, undoable_conns);
+				csch_conn_add_obj(sheet, orig_conn, newo, undoable_conns);
+			}
 		}
 		vtp0_uninit(&conn_save);
 	}
@@ -190,6 +212,8 @@ void csch_op_merge_into(csch_sheet_t *sheet, csch_cgrp_t *dst, csch_cgrp_t *src)
 	/* always move the first, don't rely on htip_next() as items are being removed from src */
 	for(e = htip_first(&src->id2obj); e != NULL; e = htip_first(&src->id2obj))
 		csch_op_move_into(sheet, dst, e->value, 1, 1);
+
+	csch_attrib_copy_all_undoable(sheet, dst, &src->attr);
 }
 
 
@@ -212,6 +236,10 @@ static void move_copy_floater_inverse(const csch_chdr_t *obj, csch_coord_t *dx,
 	if (obj->parent == &obj->sheet->direct)
 		return;
 
+	/* if there's no parent, there's no transformation possible (when copying to buffer) */
+	if (obj->parent == NULL)
+		return;
+
 	v.x = *dx; v.y = *dy;
 	csch_cgrp_inverse_matrix(&mx, obj->parent);
 
@@ -232,8 +260,14 @@ void csch_move(csch_sheet_t *sheet, csch_chdr_t *obj, csch_coord_t dx, csch_coor
 			if (!csch_grp_ref_child_moved(sheet, obj, dx, dy, undoable))
 				op->move(sheet, obj, dx, dy, undoable);
 
-			if (undoable)
+			if (undoable) {
 				csch_conn_auto_recalc(sheet, obj);
+				if (csch_obj_is_grp(obj)) {
+					csch_cgrp_t *grp = (csch_cgrp_t *)obj;
+					if (grp->role == CSCH_ROLE_WIRE_NET)
+						csch_wirenet_recalc_junctions(sheet, grp);
+				}
+			}
 			uundo_unfreeze_serial(&sheet->undo);
 			uundo_inc_serial(&sheet->undo);
 		}
