@@ -2,9 +2,9 @@
  *                            COPYRIGHT
  *
  *  cschem - modular/flexible schematics editor - sch-rnd (executable)
- *  Copyright (C) 2020,2022 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2020,2022,2023 Tibor 'Igor2' Palinkas
  *
- *  (Supported by NLnet NGI0 PET Fund in 2022)
+ *  (Supported by NLnet NGI0 PET Fund in 2022, Entrust in 2023)
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -49,9 +49,11 @@
 
 #include "util_wirenet.h"
 
+int csch_wirenet_dbg_junct = 0;
+
 void csch_wirenet_recalc_obj_conn(csch_sheet_t *sheet, csch_chdr_t *wobj)
 {
-	csch_recalc_obj_conn(sheet, wobj, CSCH_DSPLY_HUBTERM);
+	csch_recalc_obj_conn(sheet, wobj, CSCH_DSPLY_HUBTERM, CSCH_DSPLY_invalid);
 }
 
 void csch_wirenet_recalc_wirenet_conn(csch_sheet_t *sheet, csch_cgrp_t *grp)
@@ -180,21 +182,6 @@ static csch_rtree_dir_t wire_merge_cb(void *ctx_, void *obj_, const csch_rtree_b
 	return csch_RTREE_DIR_FOUND_CONT;
 }
 
-/* inverse-transform a pair of coords using parent's xform matrix */
-static void grp_inverse_xform(csch_cgrp_t *parent, csch_coord_t *x1, csch_coord_t *y1, csch_coord_t *x2, csch_coord_t *y2)
-{
-	g2d_xform_t mx;
-	g2d_vect_t v1, v2;
-
-	v1.x = *x1; v1.y = *y1;
-	v2.x = *x2; v2.y = *y2;
-	csch_cgrp_inverse_matrix(&mx, parent);
-	v1 = g2d_xform_vect2vect(mx, v1);
-	v2 = g2d_xform_vect2vect(mx, v2);
-	*x1 = v1.x; *y1 = v1.y;
-	*x2 = v2.x; *y2 = v2.y;
-}
-
 /* merges any lines that connect with "line" from within the same wirenet and
    has the same direction; line is never removed */
 csch_inline void csch_wirenet_recalc_merge(csch_sheet_t *sheet, csch_line_t *line)
@@ -259,7 +246,8 @@ csch_inline void csch_wirenet_recalc_merge(csch_sheet_t *sheet, csch_line_t *lin
 		   wirenet's coord system */
 		x1 = min.x; y1 = min.y;
 		x2 = max.x; y2 = max.y;
-		grp_inverse_xform(ctx.wirenet, &x1, &y1, &x2, &y2);
+
+		csch_cgrp_vinverse_xform(ctx.wirenet, &x1, &y1, &x2, &y2, NULL);
 		csch_line_modify(sheet, line, &x1, &y1, &x2, &y2, 1, 0);
 
 		/* remove rest of the lines */
@@ -367,7 +355,11 @@ int csch_wirenet_newline_merge_parent(csch_sheet_t *sheet, csch_coord_t x1, csch
 		g0 = adj_grp.array[0];
 		for(n = 0; n < adj_grp.used; n++) {
 			csch_cgrp_t *g = adj_grp.array[n];
-			const char *nn = csch_attrib_get_str(&g->attr, "name");
+			const char *nn;
+
+			if (g == NULL)
+				continue;
+			nn = csch_attrib_get_str(&g->attr, "name");
 			if (nn == NULL)
 				continue;
 			if (netname == NULL) {
@@ -392,7 +384,7 @@ int csch_wirenet_newline_merge_parent(csch_sheet_t *sheet, csch_coord_t x1, csch
 		parent->role = 0; /* disable side effects during the merge */
 		for(n = 0; n < adj_grp.used; n++) {
 			csch_cgrp_t *g = adj_grp.array[n];
-			if (g != g0) {
+			if ((g != NULL) && (g != g0)) {
 				g->role = 0;
 				csch_op_merge_into(sheet, parent, g);
 				g->role = CSCH_ROLE_WIRE_NET;
@@ -490,7 +482,8 @@ static void csch_wirenet_recalc_merges(csch_sheet_t *sheet, csch_cgrp_t *grp)
 			agrp->role = 0;
 			csch_op_merge_into(sheet, grp, agrp);
 			agrp->role = CSCH_ROLE_WIRE_NET;
-			csch_op_remove(sheet, &agrp->hdr);
+			if (agrp->hdr.parent != NULL)
+				csch_op_remove(sheet, &agrp->hdr);
 		}
 		grp->role = CSCH_ROLE_WIRE_NET;
 
@@ -557,6 +550,7 @@ csch_chdr_t *csch_wirenet_draw(csch_sheet_t *sheet, csch_comm_str_t stroke_name,
 	   existing line without extending it */
 	line = csch_wirenet_find_existing(sheet, x1, y1, x2, y2);
 	if (line != NULL) {
+		uundo_unfreeze_serial(&sheet->undo);
 		rnd_message(RND_MSG_WARNING, "not drawing new wire that fully overlaps with existing wire\n");
 		return NULL;
 	}
@@ -573,8 +567,8 @@ csch_chdr_t *csch_wirenet_draw(csch_sheet_t *sheet, csch_comm_str_t stroke_name,
 
 	/* transform the coords of the new line to undo the parent group's
 	   transformations */
-	grp_inverse_xform(parent, &x1, &y1, &x2, &y2);
-
+		csch_cgrp_vinverse_xform(parent, &x1, &y1, NULL);
+		csch_cgrp_vinverse_xform(parent, &x2, &y2, NULL);
 
 	line = (csch_line_t *)csch_op_create(sheet, parent, CSCH_CTYPE_LINE);
 	line->spec.p1.x = x1; line->spec.p1.y = y1;
@@ -583,8 +577,18 @@ csch_chdr_t *csch_wirenet_draw(csch_sheet_t *sheet, csch_comm_str_t stroke_name,
 	csch_line_update(sheet, line, 1);
 	csch_cgrp_bbox_update(sheet, parent);
 
-	if (merge_res > 0)
+	if (merge_res > 0) {
+		int lock_save = parent->wirenet_split_lock;
+
+		/* no split possible by drawing a new line; without disabling it, line ptr
+		   can change and that will break the calls below. Example:
+		   work/bug_files/merge* @ r6498 where recalc_merge() fails because line
+		   already has parent==NULL because recalc_junction() temporarily broke
+		   up the net */
+		parent->wirenet_split_lock = 1;
 		csch_wirenet_recalc_junctions(sheet, parent);
+		parent->wirenet_split_lock = lock_save;
+	}
 
 	csch_wirenet_recalc_obj_conn(sheet, &line->hdr);
 	csch_wirenet_recalc_merge(sheet, line); /* must be the last */
@@ -666,10 +670,18 @@ static void map_wirenet_crossings(crmap_t *map, csch_sheet_t *sheet, csch_cgrp_t
 	else
 		map->has_segmap = 0;
 
+	if (csch_wirenet_dbg_junct) rnd_trace("^^^ map_wirenet_crossings\n");
 	for(e = htip_first(&wirenet->id2obj); e != NULL; e = htip_next(&wirenet->id2obj, e)) {
 		csch_chdr_t *obj = e->value;
 		int junc = csch_obj_is_junction(obj);
 
+		if (csch_wirenet_dbg_junct) rnd_trace(" obj ID=#%ld junc=%d", obj->oid, junc);
+		if (obj->type == CSCH_CTYPE_LINE) {
+			csch_line_t *lin = e->value;
+			if (csch_wirenet_dbg_junct) rnd_trace(" {%ld %ld    %ld %ld}", lin->inst.c.p1.x, lin->inst.c.p1.y, lin->inst.c.p2.x, lin->inst.c.p2.y);
+		}
+		if (csch_wirenet_dbg_junct) rnd_trace("\n");
+
 		if (junc) { /* verify if an existing junction is still valid */
 			jk.x = (obj->bbox.x1 + obj->bbox.x2)/2;
 			jk.y = (obj->bbox.y1 + obj->bbox.y2)/2;
@@ -688,18 +700,20 @@ static void map_wirenet_crossings(crmap_t *map, csch_sheet_t *sheet, csch_cgrp_t
 				int lp_is_junc;
 
 				if ((pair == obj) || (pair->type != CSCH_CTYPE_LINE) || (pair->parent != wirenet)) continue;
-				len = g2d_iscp_cline_cline(&lo->spec, &lp->spec, ip, offs);
+				len = g2d_iscp_cline_cline(&lo->inst.c, &lp->inst.c, ip, offs); /* all junction coords are in sheet coord system because of bboxes */
 				lp_is_junc = csch_obj_is_junction(&lp->hdr);
+				if (csch_wirenet_dbg_junct) rnd_trace(" junc isc len=%d\n", len);
 				for(n = 0; n < len; n++) {
 					jk.x = ip[n].x;
 					jk.y = ip[n].y;
 					jv = htPo_get(&map->jl, jk);
 					if (!lp_is_junc)
 						jv.i++;
-					if ((offs[n] > 0) && (offs[n] < 1) && !lp_is_junc)
+					if ((offs[n] > 0) && (offs[n] < 1) && !lp_is_junc && (line_dir(lo) != line_dir(lp)))
 						jv.i++; /* mid-point crossing should have a junction even for only 2 participants */
 					if (jv.wire == NULL)
 						jv.wire = lo;
+					if (csch_wirenet_dbg_junct) rnd_trace("  [%d] junc find: %ld %ld i=%d\n", n, jk.x,jk.y, jv.i);
 					htPo_set(&map->jl, jk, jv);
 					if (enable_segmap)
 						map_seg_cr(map, lo, lp, segmap_junctions, lp_is_junc);
@@ -740,16 +754,24 @@ void csch_wirenet_recalc_junctions(csch_sheet_t *sheet, csch_cgrp_t *wirenet)
 
 	map_wirenet_crossings(&map, sheet, wirenet, 1, 1);
 
-	rnd_trace("junctions:\n");
+	if (csch_wirenet_dbg_junct) rnd_trace("^^^ junctions wirenet=#%ld:\n", wirenet->hdr.oid);
 	for(je = htPo_first(&map.jl); je != NULL; je = htPo_next(&map.jl, je)) {
-		rnd_trace(" %ld;%ld [%d] %p\n", je->key.x, je->key.y, je->value.i, je->value.ptr);
+		if (csch_wirenet_dbg_junct) rnd_trace(" %ld;%ld [%d] %p\n", je->key.x, je->key.y, je->value.i, je->value.ptr);
 		if ((je->value.i > 2) && (je->value.ptr == NULL)) {
 			int wseg;
+			csch_coord_t x1, y1, x2, y2;
 			csch_chdr_t *wire;
 			csch_line_t *line = (csch_line_t *)csch_op_create(sheet, wirenet, CSCH_CTYPE_LINE);
+			if (csch_wirenet_dbg_junct) rnd_trace(" create at %ld %ld\n", je->key.x, je->key.y);
 
-			line->spec.p1.x = line->spec.p2.x = je->key.x;
-			line->spec.p1.y = line->spec.p2.y = je->key.y;
+			/* all junction coords are in sheet coord system because of bboxes */
+			x1 = x2 = je->key.x;
+			y1 = y2 = je->key.y;
+			csch_cgrp_vinverse_xform(wirenet, &x1, &y1, &x2, &y2, NULL);
+			line->spec.p1.x = line->spec.p2.x = x1;
+			line->spec.p1.y = line->spec.p2.y = y1;
+
+			/* create the junction */
 			line->hdr.stroke_name = sheet->junction_pen_name;
 			csch_line_update(sheet, line, 1);
 			csch_cgrp_bbox_update(sheet, wirenet);
@@ -763,16 +785,15 @@ void csch_wirenet_recalc_junctions(csch_sheet_t *sheet, csch_cgrp_t *wirenet)
 				rnd_message(RND_MSG_ERROR, "Internal error: new junction has no segment");
 		}
 		else if ((je->value.i < 3) && (je->value.ptr != NULL)) {
-			rnd_trace("  remove!\n");
+			if (csch_wirenet_dbg_junct) rnd_trace(" remove at %ld %ld\n", je->key.x, je->key.y);
 			csch_op_remove(sheet, je->value.ptr);
 		}
 	}
 
-	rnd_trace("SegS: %d\n", map.seg_next);
+	if (csch_wirenet_dbg_junct) rnd_trace("SegS: %d\n", map.seg_next);
 	/* if we have more than one segments, that means the wirenet broke up;
 	   create new wirenets for each segment and move objects into them */
 	if (!wirenet->wirenet_split_lock && (map.seg_next > 2)) {
-		int n;
 		htpi_entry_t *e;
 		csch_cgrp_t **new_seg = calloc(sizeof(csch_cgrp_t), map.seg_next);
 
@@ -781,9 +802,11 @@ void csch_wirenet_recalc_junctions(csch_sheet_t *sheet, csch_cgrp_t *wirenet)
 			int seg = e->value;
 			csch_chdr_t *obj = e->key;
 			if ((seg > 1) && (!csch_obj_is_deleted(obj))) {
-				rnd_trace(" obj move %p -> %d\n", obj, seg);
-				if (new_seg[seg] == NULL)
+				if (csch_wirenet_dbg_junct) rnd_trace(" obj move %p -> %d\n", obj, seg);
+				if (new_seg[seg] == NULL) {
 					new_seg[seg] = csch_wirenet_new(sheet);
+					new_seg[seg]->hdr.selected = wirenet->hdr.selected;
+				}
 				csch_op_move_into(sheet, new_seg[seg], obj, 0, 0);
 			}
 		}
@@ -830,6 +853,7 @@ static void csch_wirenet_newseg2wirenets(csch_sheet_t *sheet, crmap_t *map, csch
 			wn = (csch_cgrp_t **)vtp0_get(new_wirenets, seg, 0);
 			if ((wn == NULL) || (*wn == NULL)) {
 				wn_ = csch_wirenet_new(sheet);
+				wn_->hdr.selected = line->hdr.parent->hdr.selected;
 				TODO("copy netname from line_wn");
 				vtp0_set(new_wirenets, seg, wn_);
 				wn = &wn_;
@@ -957,11 +981,9 @@ void csch_wirenet_recalc_unfreeze(csch_sheet_t *sheet)
 
 		for(e = htpi_first(&sheet->util_wirenet.recalc_wn); e != NULL; e = htpi_first(&sheet->util_wirenet.recalc_wn)) {
 			csch_cgrp_t *grp = e->key;
-			csch_wirenet_recalc_merges(sheet, grp);
 
-			sheet->util_wirenet.recalc_inhibit = 0;
+			csch_wirenet_recalc_merges(sheet, grp);
 			csch_wirenet_recalc_junctions(sheet, grp);
-			sheet->util_wirenet.recalc_inhibit = 1;
 
 			htpi_delentry(&sheet->util_wirenet.recalc_wn, e);
 		}
