@@ -28,6 +28,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
+#include <ctype.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/safe_fs.h>
 #include <librnd/core/plugins.h>
@@ -70,12 +71,10 @@ static void tdx_fprint_escape(FILE *f, const char *val)
 static const char *get_refdes(const csch_acomp_t *comp)
 {
 	const char *refdes;
-	refdes = csch_attrib_get_str(&comp->hdr.attr, "display/refdes");
-	if (refdes == NULL) refdes = csch_attrib_get_str(&comp->hdr.attr, "display/name");
-	if (refdes == NULL) refdes = csch_attrib_get_str(&comp->hdr.attr, "pcb/refdes");
-	if (refdes == NULL) refdes = csch_attrib_get_str(&comp->hdr.attr, "pcb/name");
-	if (refdes == NULL) refdes = csch_attrib_get_str(&comp->hdr.attr, "refdes");
+	refdes = csch_attrib_get_str(&comp->hdr.attr, "display/name");
+	if (refdes == NULL) refdes = csch_attrib_get_str(&comp->hdr.attr, "display/refdes");
 	if (refdes == NULL) refdes = csch_attrib_get_str(&comp->hdr.attr, "name");
+	if (refdes == NULL) refdes = csch_attrib_get_str(&comp->hdr.attr, "refdes");
 /*	if (refdes == NULL) refdes = comp->name; No name means do not export */
 	return refdes;
 }
@@ -83,8 +82,9 @@ static const char *get_refdes(const csch_acomp_t *comp)
 static const char *get_pinnum(const csch_aport_t *port)
 {
 	const char *pinnum;
-	pinnum = csch_attrib_get_str(&port->hdr.attr, "display/pinnum");
-	if (pinnum == NULL) pinnum = csch_attrib_get_str(&port->hdr.attr, "pcb/pinnum");
+	pinnum = csch_attrib_get_str(&port->hdr.attr, "display/name");
+	if (pinnum == NULL) pinnum = csch_attrib_get_str(&port->hdr.attr, "display/pinnum");
+	if (pinnum == NULL) pinnum = csch_attrib_get_str(&port->hdr.attr, "name");
 	if (pinnum == NULL) pinnum = csch_attrib_get_str(&port->hdr.attr, "pinnum");
 	if (pinnum == NULL) pinnum = port->name;
 	return pinnum;
@@ -94,7 +94,6 @@ static const char *get_netname(const csch_anet_t *net)
 {
 	const char *netname;
 	netname = csch_attrib_get_str(&net->hdr.attr, "display/name");
-	if (netname == NULL) netname = csch_attrib_get_str(&net->hdr.attr, "pcb/name");
 	if (netname == NULL) netname = csch_attrib_get_str(&net->hdr.attr, "name");
 	if (netname == NULL) netname = net->name;
 	return netname;
@@ -119,7 +118,7 @@ const char *get_alt_attr(const csch_attribs_t *attribs, ...)
 	return val;
 }
 
-static void export_attr(FILE *f, const char *cmd, const char *key, const char *val)
+static void export_attr(FILE *f, const char *cmd, const char *key, const char *val, const char *val2)
 {
 	if ((key == NULL) || (val == NULL))
 		return;
@@ -127,6 +126,10 @@ static void export_attr(FILE *f, const char *cmd, const char *key, const char *v
 	tdx_fprint_escape(f, key);
 	fputc(' ', f);
 	tdx_fprint_escape(f, val);
+	if (val2 != NULL) {
+		fputc(' ', f);
+		tdx_fprint_escape(f, val2);
+	}
 	fputc('\n', f);
 }
 
@@ -140,11 +143,30 @@ static void tdx_export_comps(FILE *f, csch_abstract_t *abs)
 		if (refdes == NULL)
 			continue;
 
-		export_attr(f, "footprint", refdes, get_alt_attr(&comp->hdr.attr, "display/footprint", "pcb/footprint", "footprint", NULL));
-		export_attr(f, "value", refdes, get_alt_attr(&comp->hdr.attr, "display/value", "pcb/value", "value", NULL));
-		export_attr(f, "device", refdes, get_alt_attr(&comp->hdr.attr, "display/device", "pcb/device", "device", NULL));
+		export_attr(f, "footprint", refdes, get_alt_attr(&comp->hdr.attr, "display/footprint", "footprint", NULL), NULL);
+		export_attr(f, "value", refdes, get_alt_attr(&comp->hdr.attr, "display/value", "value", NULL), NULL);
+		export_attr(f, "device", refdes, get_alt_attr(&comp->hdr.attr, "display/device", "device", NULL), NULL);
 		for(p = htsp_first(&comp->ports); p != NULL; p = htsp_next(&comp->ports, p)) {
-			export_attr(f, "pinname", refdes, get_alt_attr(&comp->hdr.attr, "display/name", "pcb/name", NULL));
+			const csch_aport_t *port = p->value;
+			const char *pinnum = get_pinnum(port);
+			const char *pinname = get_alt_attr(&port->hdr.attr, "display/pinname", "name", NULL);
+
+			if (strpbrk(pinnum, " \t") != NULL) { /* space separated list of pinnums */
+				char *s, *next, *tmp = rnd_strdup(pinnum);
+				for(s = tmp; s != NULL; s = next) {
+					next = strpbrk(s, " \t");
+					if (next != NULL) {
+						*next = '\0';
+						next++;
+					}
+					while(isspace(*s)) s++;
+					if (*s == '\0') break;
+					export_attr(f, "pinname1", refdes, s, pinname);
+				}
+				free(tmp);
+			}
+			else
+				export_attr(f, "pinname2", refdes, pinnum, pinname);
 		}
 	}
 }
@@ -153,6 +175,8 @@ static void tdx_export_nets(FILE *f, csch_abstract_t *abs)
 {
 	htsp_entry_t *e;
 	long n;
+	char *s, *next;
+	gds_t tmp = {0};
 
 	for(e = htsp_first(&abs->nets); e != NULL; e = htsp_next(&abs->nets, e)) {
 		csch_anet_t *net = e->value;
@@ -181,22 +205,40 @@ static void tdx_export_nets(FILE *f, csch_abstract_t *abs)
 				continue;
 			}
 
-			fprintf(f, "conn ");
-			tdx_fprint_escape(f, netname);
-			fputc(' ', f);
-			tdx_fprint_escape(f, refdes);
-			fputc(' ', f);
-			tdx_fprint_escape(f, pinnum);
-			fputc('\n', f);
+			/* split up pinnum at space and create one or more conn lines connecting
+			   each pin to the given net */
+			tmp.used = 0;
+			gds_append_str(&tmp, pinnum);
+			for(s = tmp.array; s != NULL; s = next) {
+				/* strip leading whitespace and seps */
+				while(isspace(*s)) s++;
+				if (*s == '\0')
+					break;
+
+				/* split string at next sep */
+				next = strpbrk(s, " \t\r\n");
+				if (next != NULL) {
+					*next = '\0';
+					next++;
+				}
+				fprintf(f, "conn ");
+				tdx_fprint_escape(f, netname);
+				fputc(' ', f);
+				tdx_fprint_escape(f, refdes);
+				fputc(' ', f);
+				tdx_fprint_escape(f, s);
+				fputc('\n', f);
+			}
 		}
 	}
+	gds_uninit(&tmp);
 }
 
 
-static int tdx_export_project(const char *fn, const char *fmt, csch_abstract_t *abs)
+static int tdx_export_project_abst(const char *fn, const char *fmt, csch_abstract_t *abs)
 {
 	TODO("get hidlib as an arg")
-	rnd_hidlib_t *hidlib = NULL;
+	rnd_design_t *hidlib = NULL;
 	FILE *f = rnd_fopen(hidlib, fn, "w");
 	if (f == NULL)
 		return -1;
@@ -231,7 +273,7 @@ int pplg_init_export_tedax(void)
 
 	etdx_net.name = "export to tEDAx";
 	etdx_net.export_prio = tdx_export_prio;
-	etdx_net.export_project = tdx_export_project;
+	etdx_net.export_project_abst = tdx_export_project_abst;
 	etdx_net.ext_export_project = ".tdx";
 	csch_plug_io_register(&etdx_net);
 
@@ -240,7 +282,7 @@ int pplg_init_export_tedax(void)
 
 	tedax_hid.struct_size = sizeof(rnd_hid_t);
 	tedax_hid.name = "tedax";
-	tedax_hid.description = "Exports tEDAx netlist";
+	tedax_hid.description = "Exports project's tEDAx netlist";
 	tedax_hid.exporter = 1;
 
 	tedax_hid.get_export_options = tedax_get_export_options;
