@@ -34,14 +34,15 @@
 #include <librnd/core/event.h>
 #include <librnd/core/actions.h>
 #include <librnd/core/hidlib.h>
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/compat_fs.h>
 #include <librnd/core/globalconst.h>
 #include <librnd/core/conf_hid.h>
-#include <librnd/core/tool.h>
+#include <librnd/hid/tool.h>
 #include <librnd/core/plugins.h>
 #include <librnd/plugins/lib_hid_common/toolbar.h>
+#include <librnd/plugins/lib_hid_common/dlg_export.h>
 
 #include <libcschem/search.h>
 #include <libcschem/event.h>
@@ -49,6 +50,8 @@
 #include <sch-rnd/search.h>
 #include <sch-rnd/funchash_core.h>
 #include <sch-rnd/multi.h>
+#include <sch-rnd/sheet.h>
+#include <sch-rnd/export.h>
 
 #include "layersel.h"
 #include "sheetsel.h"
@@ -65,6 +68,36 @@ static const char *infobar_cookie = "sch_rnd_gui/infobar";
 static const char *sch_rnd_gui_cookie = "sch_rnd_gui";
 static const char *title_cookie = "lib_hid_pcbui/title";
 
+static void sch_dwg_area_edit_ev(rnd_design_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[])
+{
+	csch_sheet_t *sheet = (csch_sheet_t *)hidlib;
+
+	if (sheet->bbox_changed) {
+		rnd_coord_t x1, y1, x2, y2;
+		rnd_coord_t min_x = sch_rnd_sheet_attr_crd(sheet, "drawing_min_width", 0);
+		rnd_coord_t min_y = sch_rnd_sheet_attr_crd(sheet, "drawing_min_height", 0);
+
+		sheet->bbox_changed = 0;
+
+
+		x1 = C2P(sheet->bbox.x1);
+		y1 = C2P(sheet->bbox.y1);
+		x2 = C2P(RND_MAX(min_x, sheet->bbox.x2));
+		y2 = C2P(RND_MAX(min_y, sheet->bbox.y2));
+
+		if ((sheet->hidlib.dwg.X1 == x1) && (sheet->hidlib.dwg.Y1 == y1) && (sheet->hidlib.dwg.X2 == x2) && (sheet->hidlib.dwg.Y2 == y2))
+			return;
+
+		sch_rnd_sheet_warn_size(sheet, sheet->bbox.x1, sheet->bbox.y1, RND_MAX(min_x, sheet->bbox.x2), RND_MAX(min_y, sheet->bbox.y2));
+
+		sheet->hidlib.dwg.X1 = x1;
+		sheet->hidlib.dwg.Y1 = y1;
+		sheet->hidlib.dwg.X2 = x2;
+		sheet->hidlib.dwg.Y2 = y2;
+	}
+}
+
+
 #define NOGUI() \
 do { \
 	if ((rnd_gui == NULL) || (!rnd_gui->gui)) { \
@@ -80,7 +113,7 @@ const char csch_acts_Popup[] = "Popup(MenuName, [obj-type])";
 const char csch_acth_Popup[] = "Bring up the popup menu specified by MenuName, optionally modified with the object type under the cursor.\n";
 fgw_error_t csch_act_Popup(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hidlib = RND_ACT_HIDLIB;
+	rnd_design_t *hidlib = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hidlib;
 	char name[256], name2[256], wnname[256];
 	const char *tn = NULL, *a0, *a1 = NULL, *misc = "misc";
@@ -178,7 +211,7 @@ static fgw_error_t csch_act_Load(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	static char *last_project = NULL, *last_layer = NULL;
 	const char *function = "Sheet";
 	char *name = NULL;
-	rnd_hidlib_t *hl = RND_ACT_HIDLIB;
+	rnd_design_t *hl = RND_ACT_DESIGN;
 	static const char *flt_any[] = {"*", "*.*", NULL};
 
 	if (last_layer == NULL)     last_layer = dup_cwd();
@@ -205,7 +238,7 @@ TODO("loadfrom action");
 			{"all",    NULL, flt_any},
 			{NULL, NULL, NULL}
 		};
-		name = rnd_gui->fileselect(rnd_gui, "Load sheet", "Import a sheet from file", last_layer, ".rs", flt, "sheet", RND_HID_FSD_READ, NULL);
+		name = rnd_hid_fileselect(rnd_gui, "Load sheet", "Import a sheet from file", last_layer, ".rs", flt, "sheet", RND_HID_FSD_READ, NULL);
 	}
 	else if (rnd_strcasecmp(function, "Project") == 0) {
 		static const char *pat_prj[] = {"project.lht", NULL};
@@ -214,7 +247,7 @@ TODO("loadfrom action");
 			{"all",    NULL, flt_any},
 			{NULL, NULL, NULL}
 		};
-		name = rnd_gui->fileselect(rnd_gui, "Load a project file", "load project (all layers) from file", last_project, ".lht", flt, "project", RND_HID_FSD_READ, NULL);
+		name = rnd_hid_fileselect(rnd_gui, "Load a project file", "load project (all layers) from file", last_project, ".lht", flt, "project", RND_HID_FSD_READ, NULL);
 	}
 	else {
 		rnd_message(RND_MSG_ERROR, "Invalid subcommand for Load(): '%s'\n", function);
@@ -233,20 +266,26 @@ TODO("loadfrom action");
 	return 0;
 }
 
-static int save_as(rnd_hidlib_t *hl, int what, int as, const char *name)
+static int save_as(rnd_design_t *hl, int what, int as, const char *name)
 {
 	csch_sheet_t *sheet = (csch_sheet_t *)hl;
 	const char *prev_fn, *whats, *prev_loadfn;
-	char *free_me = NULL;
+	char *free_me = NULL, *free_me2 = NULL;
 	int need_fsd, save_res = -1;
+	int append_rs = 0;
+
+	if ((hl->fullpath != NULL) && (hl->fullpath[0] == '<')) {
+		as = 1; /* force save-as on <none> */
+		append_rs = 1;
+	}
 
 	/* figure previous file name and whether we need to invoke FSD */
 	if (as) {
 		if (name == NULL) {
-			prev_fn = sheet->hidlib.filename;
+			prev_fn = sheet->hidlib.fullpath;
 			if (prev_fn == NULL)
 				prev_fn = free_me = dup_cwd();
-			prev_loadfn = sheet->loadname;
+			prev_loadfn = sheet->hidlib.loadname;
 			if (prev_loadfn == NULL)
 				prev_loadfn = prev_fn;
 			need_fsd = 1;
@@ -257,7 +296,7 @@ static int save_as(rnd_hidlib_t *hl, int what, int as, const char *name)
 		}
 	}
 	else {
-		prev_fn = name = sheet->hidlib.filename;
+		prev_fn = name = sheet->hidlib.fullpath;
 		if (name == NULL) {
 			prev_fn = free_me = dup_cwd();
 			need_fsd = 1;
@@ -265,11 +304,14 @@ static int save_as(rnd_hidlib_t *hl, int what, int as, const char *name)
 		else
 			need_fsd = 0;
 
-		prev_loadfn = sheet->loadname;
+		prev_loadfn = sheet->hidlib.loadname;
 		if (prev_loadfn == NULL)
 			prev_loadfn = prev_fn;
 	}
 
+	if (append_rs)
+		prev_loadfn = free_me2 = rnd_concat(prev_loadfn, ".rs", NULL);
+
 	/* invoke FSD if needed */
 	switch(what) {
 		case F_Sheet:
@@ -295,16 +337,13 @@ static int save_as(rnd_hidlib_t *hl, int what, int as, const char *name)
 	else if (rnd_conf.rc.verbose)
 		fprintf(stderr, "Save: SaveTo(%s, ...) cancelled\n", whats);
 
-	if (as && (save_res == 0)) {
-		free(sheet->loadname);
-		sheet->loadname = rnd_strdup(name);
-	}
-
 	free(free_me);
-	return 0;
+	free(free_me2);
+	return save_res;
 
 	error:;
 	free(free_me);
+	free(free_me2);
 	return 1;
 }
 
@@ -314,7 +353,7 @@ static const char csch_acts_SaveAs[] = "SaveAs(Project|Sheet, [filename])";
 static const char csch_acth_SaveAs[] = "Save a project or a schmeatics sheet using a new file name.";
 static fgw_error_t csch_act_Save(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hl = RND_ACT_HIDLIB;
+	rnd_design_t *hl = RND_ACT_DESIGN;
 	const char *actname = argv[0].val.func->name;
 	int as = (actname[4] == 'a') || (actname[4] == 'A');
 	int what = F_Sheet;
@@ -347,6 +386,18 @@ static fgw_error_t csch_act_SwitchRelative(fgw_arg_t *res, int argc, fgw_arg_t *
 	return 0;
 }
 
+const char csch_acts_ExportProjectDialog[] = "ExportDialog()\n";
+const char csch_acth_ExportProjectDialog[] = "Open the export dialog for exporting the project (all sheets of the project).";
+fgw_error_t csch_act_ExportProjectDialog(fgw_arg_t *res, int argc, fgw_arg_t *argv)
+{
+	static sch_rnd_export_appspec_t appspec = {0};
+
+	appspec.exp_prj = 1;
+	rnd_dlg_export("Export project to file(s)", 1, 0, RND_ACT_DESIGN, &appspec);
+	return 0;
+}
+
+
 #include "edit_act.c"
 
 static rnd_action_t sch_rnd_gui_action_list[] = {
@@ -358,7 +409,8 @@ static rnd_action_t sch_rnd_gui_action_list[] = {
 	{"SwitchRelative", csch_act_SwitchRelative, csch_acth_SwitchRelative, csch_acts_SwitchRelative},
 	EDIT_ACTS
 	{"Popup", csch_act_Popup, csch_acth_Popup, csch_acts_Popup},
-	{"StatusSetText", csch_act_StatusSetText, csch_acth_StatusSetText, csch_acts_StatusSetText}
+	{"StatusSetText", csch_act_StatusSetText, csch_acth_StatusSetText, csch_acts_StatusSetText},
+	{"ExportProjectDialog", csch_act_ExportProjectDialog, csch_acth_ExportProjectDialog, csch_acts_ExportProjectDialog},
 };
 
 int pplg_check_ver_gui(int ver_needed) { return 0; }
@@ -376,18 +428,15 @@ void pplg_uninit_gui(void)
 	rnd_conf_hid_unreg(infobar_cookie);
 }
 
-TODO("librnd: API change requred so conf change gets hidlib passed")
-extern rnd_hidlib_t *csch_hidlib;
-
-static void gui_conf_val_change_post(rnd_conf_native_t *cfg, int arr_idx)
+static void gui_conf_val_change_post(rnd_conf_native_t *cfg, int arr_idx, void *user_data)
 {
 	if (strncmp(cfg->hash_path, "editor/style/tool", 17) == 0) {
 		/* re-activate current tool to get this applied */
-		rnd_tool_chg_mode(csch_hidlib);
+		rnd_tool_chg_mode(rnd_multi_get_current());
 	}
 }
 
-static rnd_conf_hid_id_t install_events(const char *cookie, const char *paths[], rnd_conf_hid_callbacks_t cb[], void (*update_cb)(rnd_conf_native_t*,int))
+static rnd_conf_hid_id_t install_events(const char *cookie, const char *paths[], rnd_conf_hid_callbacks_t cb[], void (*update_cb)(rnd_conf_native_t*,int,void*))
 {
 	const char **rp;
 	rnd_conf_native_t *nat;
@@ -421,24 +470,27 @@ int pplg_init_gui(void)
 	rnd_event_bind(RND_EVENT_GUI_INIT, sch_rnd_sheetsel_gui_init_ev, NULL, sheetsel_cookie);
 	rnd_event_bind(RND_EVENT_GUI_INIT, sch_rnd_layersel_gui_init_ev, NULL, layersel_cookie);
 	rnd_event_bind(RND_EVENT_GUI_INIT, sch_status_gui_init_ev, NULL, layersel_cookie);
-	rnd_event_bind(CSCH_EVENT_LAYERS_CHANGED, sch_rnd_layersel_layer_chg_ev, NULL, layersel_cookie);
 	rnd_event_bind(RND_EVENT_USER_INPUT_KEY, sch_status_st_update_ev, NULL, status_cookie);
-	rnd_event_bind(RND_EVENT_BOARD_CHANGED, sch_status_st_update_ev, NULL, status_cookie);
+	rnd_event_bind(RND_EVENT_DESIGN_SET_CURRENT, sch_status_st_update_ev, NULL, status_cookie);
 	rnd_event_bind(CSCH_EVENT_SHEET_POSTLOAD, sch_status_postload_ev, NULL, status_cookie);
 	rnd_event_bind(RND_EVENT_CROSSHAIR_MOVE, sch_status_rd_update_ev, NULL, status_cookie);
 	rnd_event_bind(CSCH_EVENT_SHEET_EDITED, sch_status_rd_edit_ev, NULL, status_cookie);
+	rnd_event_bind(CSCH_EVENT_SHEET_EDITED, sch_dwg_area_edit_ev, NULL, status_cookie);
 	rnd_event_bind(RND_EVENT_GUI_INIT, pcb_infobar_gui_init_ev, NULL, infobar_cookie);
-	rnd_event_bind(RND_EVENT_BOARD_CHANGED, pcb_infobar_brdchg_ev, NULL, infobar_cookie);
+	rnd_event_bind(RND_EVENT_DESIGN_SET_CURRENT, pcb_infobar_brdchg_ev, NULL, infobar_cookie);
 	rnd_event_bind(CSCH_EVENT_SHEET_POSTSAVE, pcb_infobar_postsave_ev, NULL, infobar_cookie);
 	rnd_event_bind(CSCH_EVENT_SHEET_POSTLOAD, pcb_infobar_postload_ev, NULL, infobar_cookie);
 	rnd_event_bind(CSCH_EVENT_PRJ_VIEW_ACTIVATED, sch_status_view_activated_ev, NULL, infobar_cookie);
-	rnd_event_bind(RND_EVENT_BOARD_CHANGED, sch_title_board_changed_ev, NULL, title_cookie);
-	rnd_event_bind(RND_EVENT_BOARD_CHANGED, sch_sheetsel_board_changed_ev, NULL, sheetsel_cookie);
-	rnd_event_bind(RND_EVENT_BOARD_FN_CHANGED, sch_title_board_changed_ev, NULL, title_cookie);
-	rnd_event_bind(RND_EVENT_BOARD_META_CHANGED, sch_title_meta_changed_ev, NULL, title_cookie);
+	rnd_event_bind(RND_EVENT_DESIGN_SET_CURRENT, sch_title_board_changed_ev, NULL, title_cookie);
+	rnd_event_bind(RND_EVENT_DESIGN_SET_CURRENT, sch_sheetsel_board_changed_ev, NULL, sheetsel_cookie);
+	rnd_event_bind(RND_EVENT_DESIGN_FN_CHANGED, sch_title_board_changed_ev, NULL, title_cookie);
+	rnd_event_bind(RND_EVENT_DESIGN_META_CHANGED, sch_title_meta_changed_ev, NULL, title_cookie);
+	rnd_event_bind(RND_EVENT_DESIGN_META_CHANGED, sch_sheetsel_meta_changed_ev, NULL, title_cookie);
 	rnd_event_bind(RND_EVENT_GUI_INIT, sch_title_gui_init_ev, NULL, title_cookie);
 	rnd_event_bind(RND_EVENT_LOAD_POST, sch_sheetsel_load_post_ev, NULL, sheetsel_cookie);
+	rnd_event_bind(RND_EVENT_DESIGN_FN_CHANGED, sch_sheetsel_fn_changed_ev, NULL, sheetsel_cookie);
 	rnd_event_bind(CSCH_EVENT_SHEET_POSTUNLOAD, sch_sheetsel_unload_post_ev, NULL, sheetsel_cookie);
+	rnd_event_bind(CSCH_EVENT_LAYERVIS_CHANGED, sch_rnd_layersel_vis_chg_ev, NULL, layersel_cookie);
 
 	RND_REGISTER_ACTIONS(sch_rnd_gui_action_list, sch_rnd_gui_cookie);
 
