@@ -2,9 +2,9 @@
  *                            COPYRIGHT
  *
  *  cschem - modular/flexible schematics editor - gEDA file format support
- *  Copyright (C) 2022 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2022, 2023 Tibor 'Igor2' Palinkas
  *
- *  (Supported by NLnet NGI0 PET Fund in 2022)
+ *  (Supported by NLnet NGI0 PET Fund in 2022, Entrust in 2023)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -651,8 +651,6 @@ static int read_path(read_ctx_t *ctx, csch_cgrp_t *parent)
 	dbg_printf("path of %ld:\n", numlines);
 
 	if (numlines > 0) {
-		csch_source_arg_t *src;
-
 		penname = read_alloc_pen(ctx, parent, color, width, capstyle, dashstyle, dashlength, dashspace);
 		poly = csch_alien_mkpoly(&ctx->alien, parent, penname, (filltype > 0) ? penname : NULL);
 		ctx->last = poly;
@@ -861,7 +859,7 @@ static void remove_orig_attr_by_text(read_ctx_t *ctx, csch_cgrp_t *sym, const ch
 	free(template);
 }
 
-static csch_cgrp_t *load_sym(rnd_hidlib_t *hidlib, const char *fn, csch_sheet_t *sheet);
+static csch_cgrp_t *load_sym(rnd_design_t *hidlib, const char *fn, csch_sheet_t *sheet);
 
 /* Move over all attributes and properites from src sym to dst sym and remove
    src sym */
@@ -901,7 +899,7 @@ static void last_comp_move_fields(read_ctx_t *ctx, csch_cgrp_t *dst, csch_cgrp_t
 
 			remove_orig_attr_by_text(ctx, dst, t->text);
 
-			newt = csch_text_dup(ctx->sheet, dst, t, 0);
+			newt = csch_text_dup(ctx->sheet, dst, t, 0, 0);
 			newt->spec1.x -= dst->x;
 			newt->spec1.y -= dst->y;
 			csch_text_update(ctx->sheet, newt, 1);
@@ -1101,6 +1099,7 @@ static int read_any(read_ctx_t *ctx, csch_cgrp_t *parent, int in_emcomp)
 	return res;
 }
 
+#include "read_postproc.c"
 
 /*** file level parsing and entry points ***/
 
@@ -1108,6 +1107,17 @@ static void alien_setup(read_ctx_t *ctx)
 {
 	ctx->alien.sheet = ctx->sheet;
 	ctx->alien.coord_factor = io_geda_conf.plugins.io_geda.coord_mult;
+	ctx->alien.fmt_prefix = "io_geda";
+}
+
+static int io_geda_postproc(read_ctx_t *ctx)
+{
+	if (io_geda_conf.plugins.io_geda.emulate_text_ang_180) {
+		csch_cgrp_update(ctx->sheet, &ctx->sheet->direct, 1);
+		csch_alien_postproc_text_autorot(&ctx->alien, &ctx->sheet->direct, 1, 0);
+	}
+	postproc_symbols(ctx);
+	return csch_alien_postproc_sheet(&ctx->alien);
 }
 
 int io_geda_load_sheet(FILE *f, const char *fn, const char *fmt, csch_sheet_t *dst)
@@ -1148,6 +1158,12 @@ int io_geda_load_sheet(FILE *f, const char *fn, const char *fmt, csch_sheet_t *d
 	if (ctx.buffer_dirty)
 		sch_rnd_buffer_clear(sch_rnd_buffer[SCH_RND_BUFFER_SCRATCH]);
 
+	if (res == 0)
+		res = io_geda_postproc(&ctx);
+
+	if ((res == 0) && io_geda_conf.plugins.io_geda.auto_normalize)
+		csch_alien_postproc_normalize(&ctx.alien);
+
 	uninit_sym_hash(&ctx);
 
 	return res;
@@ -1199,7 +1215,7 @@ static csch_cgrp_t *load_sym_(read_ctx_t *ctx)
 }
 
 /* Load symbol from fn into sheet and return the group; sheet is typically a buffer */
-static csch_cgrp_t *load_sym(rnd_hidlib_t *hidlib, const char *fn, csch_sheet_t *sheet)
+static csch_cgrp_t *load_sym(rnd_design_t *hidlib, const char *fn, csch_sheet_t *sheet)
 {
 	read_ctx_t ctx = {0};
 	csch_cgrp_t *res;
@@ -1218,6 +1234,9 @@ static csch_cgrp_t *load_sym(rnd_hidlib_t *hidlib, const char *fn, csch_sheet_t
 
 	res = load_sym_(&ctx);
 
+	if (res == 0)
+		io_geda_postproc(&ctx);
+
 	fclose(ctx.f);
 
 	return res;
