@@ -583,7 +583,7 @@ static int io_lihata_save_sheet_or_buff(const char *fn, const char *fmt, const c
 	write_ctx_t ctx;
 
 	ctx.sheet = dst;
-	ctx.f = rnd_fopen((rnd_hidlib_t *)&dst->hidlib, fn, "w");
+	ctx.f = rnd_fopen((rnd_design_t *)&dst->hidlib, fn, "w");
 	if (ctx.f == NULL)
 		return -1;
 
@@ -615,7 +615,7 @@ int io_lihata_save_grp(const char *fn, const char *fmt, const csch_cgrp_t *src)
 	write_ctx_t ctx;
 
 	ctx.sheet = src->hdr.sheet;
-	ctx.f = rnd_fopen((rnd_hidlib_t *)&ctx.sheet->hidlib, fn, "w");
+	ctx.f = rnd_fopen((rnd_design_t *)&ctx.sheet->hidlib, fn, "w");
 	if (ctx.f == NULL)
 		return -1;
 
