@@ -30,10 +30,14 @@
 
 typedef struct csch_alien_read_ctx_s {
 	csch_sheet_t *sheet;
+	const char *fmt_prefix; /* for postproc actions */
 	double coord_factor;
 	double ox, oy;        /* in original format coords */
 	unsigned flip_x:1;
 	unsigned flip_y:1;
+
+	/* print errors only once */
+	unsigned warned_coord_range:1;
 } csch_alien_read_ctx_t;
 
 /* Set up ctx->sheet:
@@ -71,3 +75,14 @@ void csch_alien_mkbezier(csch_alien_read_ctx_t *ctx, csch_cgrp_t *parent, double
 csch_coord_t csch_alien_coord_x(csch_alien_read_ctx_t *ctx, double crd);
 csch_coord_t csch_alien_coord_y(csch_alien_read_ctx_t *ctx, double crd);
 csch_coord_t csch_alien_coord(csch_alien_read_ctx_t *ctx, double crd);
+
+
+/* Standard alien file format postprocessing. Returns 0 on success */
+int csch_alien_postproc_sheet(csch_alien_read_ctx_t *ctx);
+
+/* Rebase origin: move sheet's bbox x1;y1 to 0;0 */
+void csch_alien_postproc_normalize(csch_alien_read_ctx_t *ctx);
+
+/* recursively emulate tune text rotation; 0 and 90 degree text is always
+   kept; 180 or 270 degree text is adjusted depending on the arguments */
+void csch_alien_postproc_text_autorot(csch_alien_read_ctx_t *ctx, csch_cgrp_t *grp, int fix180, int fix270);
