@@ -35,6 +35,7 @@
 #include <librnd/core/plugins.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/rnd_printf.h>
+#include <librnd/core/misc_util.h>
 #include <sch-rnd/funchash_core.h>
 #include <sch-rnd/crosshair.h>
 
@@ -133,10 +134,10 @@ csch_oidpath_t *prop_scope_single_obj(csch_propedit_t *pe)
 }
 
 static const char csch_acts_propset[] = "propset([scope], name, value)";
-static const char csch_acth_propset[] = "Change the named property of scope or all selected objects to/by value. Scope is documented at PropEdit().";
+static const char csch_acth_propset[] = "Change the named property of scope or all selected objects to/by value. Scope is documented at PropEdit(). Existing attributes can be renamed to value by using a name rename/a/old_name, where old_name is the current name of the attribute.";
 static fgw_error_t csch_act_propset(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hl = RND_ACT_HIDLIB;
+	rnd_design_t *hl = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hl;
 	const char *first, *name, *val;
 	csch_propedit_t ctx;
@@ -155,6 +156,15 @@ static fgw_error_t csch_act_propset(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		RND_ACT_CONVARG(3, FGW_STR, propset, name = argv[3].val.str);
 		RND_ACT_CONVARG(4, FGW_STR, propset, val = argv[4].val.str);
 	}
+	if (strcmp(first, "@") == 0) {
+		if (sheet->currobj == NULL) {
+			rnd_message(RND_MSG_ERROR, "propset: @ without sheet->currobj set\n");
+			return FGW_ERR_ARG_CONV;
+		}
+		prop_scope_add_obj(&ctx, sheet->currobj);
+		RND_ACT_CONVARG(2, FGW_STR, propset, name = argv[2].val.str);
+		RND_ACT_CONVARG(3, FGW_STR, propset, val = argv[3].val.str);
+	}
 	else if (prop_scope_add(&ctx, first, 1) == 0) {
 		RND_ACT_CONVARG(2, FGW_STR, propset, name = argv[2].val.str);
 		RND_ACT_CONVARG(3, FGW_STR, propset, val = argv[3].val.str);
@@ -176,7 +186,7 @@ static const char csch_acts_proptoggle[] = "proptoggle([scope], name, [create])"
 static const char csch_acth_proptoggle[] = "Toggle the named property of scope or all selected objects, assuming the property is boolean. Scope is documented at PropEdit(). If create is true, non-existing attributes are created as true.";
 fgw_error_t csch_act_proptoggle(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hl = RND_ACT_HIDLIB;
+	rnd_design_t *hl = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hl;
 	const char *first, *name, *create = NULL;
 	csch_propedit_t ctx;
@@ -206,7 +216,7 @@ static const char csch_acts_propget[] = "propget([scope], name, [stattype])";
 static const char csch_acth_propget[] = "Return the named property of scope or all selected objects to/by value. Scope is documented at PropEdit().";
 fgw_error_t csch_act_propget(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hl = RND_ACT_HIDLIB;
+	rnd_design_t *hl = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hl;
 	const char *first, *name, *stty = "common";
 	csch_props_t *p;
@@ -297,7 +307,7 @@ static const char csch_acts_propprint[] = "PropPrint([scope])";
 static const char csch_acth_propprint[] = "Print a property map of objects matching the scope. Scope is documented at PropEdit().";
 fgw_error_t csch_act_propprint(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hl = RND_ACT_HIDLIB;
+	rnd_design_t *hl = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hl;
 	const char *scope = NULL;
 	htsp_entry_t *e, *sorted;
@@ -329,8 +339,8 @@ fgw_error_t csch_act_propprint(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 			case CSCH_PROPT_STRING: printf("	common='%s'\n", com.string); break;
 			case CSCH_PROPT_COMMS: printf("	common='%s'\n", com.comms.str); break;
 			case CSCH_PROPT_COORD:
-				rnd_printf("	common='%$$mm'\n", com.coord);
-				rnd_printf("	min/avg/max=%$$mm/%$$mm/%$$mm\n", min.coord, avg.coord, max.coord);
+				rnd_printf("	common='%$$rc'\n", com.coord);
+				rnd_printf("	min/avg/max=%$$rc/%$$rc/%$$rc\n", min.coord, avg.coord, max.coord);
 				break;
 			case CSCH_PROPT_ANGLE:
 				rnd_printf("	common='%f'\n", com.angle);
