@@ -2,10 +2,10 @@
  *                            COPYRIGHT
  *
  *  cschem - modular/flexible schematics editor - property editor plugin
- *  Copyright (C) 2022 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2022, 2023 Tibor 'Igor2' Palinkas
  *  (Adapted from propsel code in pcb-rnd by the same author)
  *
- *  (Supported by NLnet NGI0 PET Fund in 2022)
+ *  (Supported by NLnet NGI0 PET Fund in 2022 and Entrust in 2023)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 #include <librnd/core/misc_util.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/rnd_printf.h>
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 #include <librnd/core/error.h>
 #include <librnd/core/math_helper.h>
 
@@ -151,9 +151,9 @@ static void map_attr(void *ctx, htsp_t *attrs)
 
 static void map_sheet(csch_propedit_t *ctx, csch_sheet_t *sheet)
 {
-	map_add_prop(ctx, "p/board/loadname", String, sheet->loadname);
-	map_add_prop(ctx, "p/board/filename", String, sheet->hidlib.filename);
-	map_add_prop(ctx, "p/board/loose_sym", bool, sheet->loose_sym);
+	map_add_prop(ctx, "p/sheet/loadname", String, sheet->hidlib.loadname);
+	map_add_prop(ctx, "p/sheet/filename", String, sheet->hidlib.fullpath);
+	map_add_prop(ctx, "p/sheet/loose_sym", bool, sheet->loose_sym);
 	map_attr(ctx, &sheet->direct.attr);
 }
 
@@ -266,17 +266,16 @@ static long csch_propsel_do(csch_sheet_t *sheet, csch_cgrp_t *grp, int data_in_g
 	for(e = htip_first(&grp->id2obj); e != NULL; e = htip_next(&grp->id2obj, e)) {
 		csch_chdr_t *h = e->value;
 		if (data_in_grp) {
-			TODO("implement this:")
+			TODO("rethink this: propset() should work on objects under group lock; example Erich's case on removing all pin numbers require to change them to floater so they can be removed; pcb-rnd also allows propset() to bypass group lock; pool node with the script: sch-rnd remove_pinnum")
 #if 0
 			if (floater_or_parent_group_atomic?(h))
 				continue; /* grp lock: skip any selected object within a grp */
 #endif
 		}
-		else {
-			if (csch_obj_is_grp(h)) {
-				csch_cgrp_t *grp = (csch_cgrp_t *)h;
-				res += csch_propsel_do(sheet, grp, 1, ctx, cb, udata); /* recurse to groups */
-			}
+
+		if (csch_obj_is_grp(h)) {
+			csch_cgrp_t *grp = (csch_cgrp_t *)h;
+			res += csch_propsel_do(sheet, grp, 1, ctx, cb, udata); /* recurse to groups */
 		}
 
 		if (h->selected)
@@ -358,6 +357,7 @@ static void set_attr_obj(csch_propset_ctx_t *st, csch_chdr_t *hdr)
 {
 	const char *key = st->name+2;
 	const char *orig;
+	csch_attrib_t *orig_a;
 	csch_cgrp_t *grp = (csch_cgrp_t *)hdr;
 
 	assert(csch_obj_is_grp(hdr));
@@ -367,11 +367,26 @@ static void set_attr_obj(csch_propset_ctx_t *st, csch_chdr_t *hdr)
 		return;
 	}
 
-	orig = csch_attrib_get_str(&grp->attr, key);
-	if ((orig != NULL) && (strcmp(orig, st->s) == 0))
-		return;
+	orig_a = csch_attrib_get(&grp->attr, key);
+	if ((orig_a == NULL) || orig_a->deleted)
+		orig = NULL;
+	else
+		orig = orig_a->val;
 
-	csch_attr_modify_str(st->sheet, grp, CSCH_ATP_USER_DEFAULT, key, st->s, propsel_src(), 1);
+	if (st->arename) {
+		if ((orig_a == NULL) || (strcmp(key, st->s) == 0))
+			return;
+	}
+	else {
+		if ((orig != NULL) && (strcmp(orig, st->s) == 0))
+			return;
+	}
+
+
+	if (st->arename)
+		csch_attr_modify_rename(st->sheet, grp, orig_a, st->s, propsel_src(), 1);
+	else
+		csch_attr_modify_str(st->sheet, grp, CSCH_ATP_USER_DEFAULT, key, st->s, propsel_src(), 1);
 	st->set_cnt++;
 }
 
@@ -398,8 +413,8 @@ TODO("sheet prop set");
 		    (csch_board_change_name(rnd_strdup(st->s)))) DONE;
 
 		if ((strcmp(pn, "filename") == 0)) {
-			free(pcb->hidlib.filename);
-			pcb->hidlib.filename = rnd_strdup(st->s);
+			free(pcb->hidlib.fullpath);
+			pcb->hidlib.fullpath = rnd_strdup(st->s);
 			DONE;
 		}
 #endif
@@ -755,18 +770,21 @@ int csch_propsel_set(csch_propedit_t *ctx, const char *prop, csch_propset_ctx_t
 
 int csch_propsel_set_str(csch_propedit_t *ctx, const char *prop, const char *value)
 {
-	csch_propset_ctx_t sctx;
+	csch_propset_ctx_t sctx = {0};
 	char *end;
 	const char *start;
 
 	/* sanity checks for invalid props */
+	if ((prop[0] == 'r') && (strncmp(prop, "rename/a/", 9)) == 0) {
+		sctx.arename = 1;
+		prop += 7;
+	}
+
 	if ((prop[1] != '/') || ((prop[0] != 'a') && (prop[0] != 'p'))) {
 		rnd_message(RND_MSG_ERROR, "Invalid property path: '%s':\n must start with p/ for property or a/ for attribute\n", prop);
 		return 0;
 	}
 
-	memset(&sctx, 0, sizeof(sctx));
-
 	if (value == NULL)
 		value = "";
 	sctx.s = value;
@@ -857,7 +875,7 @@ char *csch_propsel_printval(csch_props_t *p, const csch_propval_t *val)
 
 	switch(p->type) {
 		case CSCH_PROPT_STRING: return val->string == NULL ? rnd_strdup("") : rnd_strdup(val->string);
-		case CSCH_PROPT_COORD:  return rnd_strdup_printf("%ld", (long)val->coord);
+		case CSCH_PROPT_COORD:  return rnd_strdup_printf("%$rc", val->coord);
 		case CSCH_PROPT_ANGLE:  return rnd_strdup_printf("%f", val->angle);
 		case CSCH_PROPT_DOUBLE: return rnd_strdup_printf("%f", val->d);
 		case CSCH_PROPT_INT:    return rnd_strdup_printf("%d", val->i);
