@@ -33,8 +33,8 @@
 
 #include <librnd/core/actions.h>
 #include <librnd/core/conf_hid.h>
-#include <librnd/core/hidlib_conf.h>
-#include <librnd/core/hid_dad.h>
+#include <librnd/core/rnd_conf.h>
+#include <librnd/hid/hid_dad.h>
 #include <librnd/core/event.h>
 #include <librnd/core/error.h>
 #include "query.h"
@@ -468,7 +468,7 @@ static void search_append_row_cb(void *hid_ctx, void *caller_data, rnd_hid_attri
 static void search_apply_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr)
 {
 	search_ctx_t *ctx = caller_data;
-	rnd_hidlib_t *hidlib = rnd_gui->get_dad_hidlib(hid_ctx);
+	rnd_design_t *hidlib = rnd_gui->get_dad_design(hid_ctx);
 	if (ctx->dlg[ctx->wexpr_str].val.str != NULL) {
 		const char *op = search_acts[ctx->dlg[ctx->wact].val.lng];
 		long res = rnd_actionva(hidlib, "query", search_acts[ctx->dlg[ctx->wact].val.lng], ctx->dlg[ctx->wexpr_str].val.str, NULL);
@@ -610,11 +610,11 @@ static void search_close(search_ctx_t *ctx)
 }
 
 
-void sch_rnd_query_ev_preunload(rnd_hidlib_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[])
+void sch_rnd_query_ev_preunload(rnd_design_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[])
 {
 	search_ctx_t *ctx, *nextctx;
 	for(ctx = gdl_first(&searches); ctx != NULL; ctx = nextctx) {
-		rnd_hidlib_t *ctx_hl = rnd_gui->get_dad_hidlib(ctx->dlg_hid_ctx);
+		rnd_design_t *ctx_hl = rnd_gui->get_dad_design(ctx->dlg_hid_ctx);
 		nextctx = gdl_next(&searches, ctx);
 		if (ctx_hl == hidlib)
 			search_close(ctx);
