@@ -69,6 +69,12 @@ void pcb_qry_list_all_sheet(pcb_qry_val_t *lst, csch_sheet_t *sheet, csch_cmask_
 	list_grp(&lst->data.lst, &sheet->direct, 1, mask);
 }
 
+void pcb_qry_list_all_sheet_indirect(pcb_qry_val_t *lst, csch_sheet_t *sheet, csch_cmask_t mask)
+{
+	assert(lst->type == PCBQ_VT_LST);
+	list_grp(&lst->data.lst, &sheet->indirect, 1, mask);
+}
+
 void pcb_qry_list_all_grp(pcb_qry_val_t *lst, csch_cgrp_t *grp, csch_cmask_t mask)
 {
 	list_grp(&lst->data.lst, grp, 1, mask);
@@ -453,6 +459,7 @@ static int field_text(pcb_qry_exec_t *ec, csch_chdr_t *obj, pcb_qry_node_t *fld,
 		case query_fields_mirx:     PCB_QRY_RET_INT(res, t->inst_mirx);
 		case query_fields_miry:     PCB_QRY_RET_INT(res, t->inst_miry);
 		case query_fields_string:   PCB_QRY_RET_STR(res, t->text);
+		case query_fields_text:     PCB_QRY_RET_STR(res, t->text);
 		case query_fields_render:   PCB_QRY_RET_STR(res, csch_text_get_rtext(t));
 		case query_fields_dyntext:  PCB_QRY_RET_INT(res, t->dyntext);
 		case query_fields_has_bbox: PCB_QRY_RET_INT(res, t->has_bbox);
