@@ -43,6 +43,7 @@
 #include <libcschem/cnc_any_obj.h>
 #include "dlg_search.h"
 #include <librnd/core/compat_misc.h>
+#include <librnd/hid/hid.h>
 
 static const char csch_acts_query[] =
 	"query(dump, expr) - dry run: compile and dump an expression\n"
@@ -278,7 +279,7 @@ int pcb_qry_extract_defs(htsi_t *dst, const char *script)
 
 static fgw_error_t csch_act_query(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hidlib = RND_ACT_HIDLIB;
+	rnd_design_t *hidlib = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hidlib;
 	const char *cmd, *arg = NULL, *scope = NULL;
 	flagop_t sel;
@@ -361,7 +362,7 @@ static fgw_error_t csch_act_query(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		if (sel.cnt > 0) {
 			csch_sheet_set_changed(sheet, rnd_true);
 			if (RND_HAVE_GUI_ATTR_DLG)
-				rnd_hid_redraw(PCB);
+				rnd_hid_redraw(hidlib);
 		}
 		RND_ACT_IRES(0);
 		return 0;
@@ -379,7 +380,7 @@ static fgw_error_t csch_act_query(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		if (sel.cnt > 0) {
 			csch_sheet_set_changed(sheet, rnd_true);
 			if (RND_HAVE_GUI_ATTR_DLG)
-				rnd_hid_redraw(PCB);
+				rnd_hid_redraw(hidlib);
 		}
 		RND_ACT_IRES(0);
 		return 0;
@@ -409,7 +410,7 @@ static fgw_error_t csch_act_query(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		if (pcb_qry_run_script(NULL, sheet, arg, scope, view_cb, &arr) >= 0) {
 			args[1].type = FGW_STR; args[1].val.str = "objarr";
 			fgw_ptr_reg(&rnd_fgw, &args[2], CSCH_PTR_DOMAIN_COBJ_ARR, FGW_PTR | FGW_STRUCT, &arr);
-			rnd_actionv_bin(RND_ACT_HIDLIB, "TreeDialog", &ares, 3, args);
+			rnd_actionv_bin(RND_ACT_DESIGN, "TreeDialog", &ares, 3, args);
 			fgw_ptr_unreg(&rnd_fgw, &args[2], CSCH_PTR_DOMAIN_COBJ_ARR);
 			vtp0_uninit(&arr);
 			RND_ACT_IRES(0);
@@ -529,7 +530,7 @@ static const char csch_acts_QueryObj[] = "QueryObj(idpath, [.fieldname|fieldID])
 static const char csch_acth_QueryObj[] = "Return the value of a field of an object, addressed by the object's idpath and the field's name or precompiled ID. Returns NIL on error.";
 static fgw_error_t csch_act_QueryObj(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hidlib = RND_ACT_HIDLIB;
+	rnd_design_t *hidlib = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hidlib;
 	csch_oidpath_t *idp;
 	pcb_qry_node_t *fld = NULL;
