@@ -35,9 +35,9 @@
 #include <librnd/core/event.h>
 #include <librnd/core/actions.h>
 #include <librnd/core/hidlib.h>
-#include <librnd/core/hidlib_conf.h>
-#include <librnd/core/hid_dad.h>
-#include <librnd/core/hid_dad_tree.h>
+#include <librnd/core/rnd_conf.h>
+#include <librnd/hid/hid_dad.h>
+#include <librnd/hid/hid_dad_tree.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/compat_fs.h>
 #include <librnd/core/globalconst.h>
@@ -159,6 +159,7 @@ static void attr2dlg_arr(attrdlg_ctx_t *ctx, csch_attrib_t *a)
 	rnd_dad_tree_clear(tree);
 
 	/* add all items recursively */
+	cell[1] = NULL;
 	for(n = 0; n < a->arr.used; n++) {
 		cell[0] = rnd_strdup(a->arr.array[n]);
 		r = rnd_dad_tree_append(attr, NULL, cell);
@@ -383,15 +384,9 @@ static void attr_val_set_meta_and_str(attrdlg_ctx_t *ctx, csch_attrib_t *a, int
 		return; /* do not allow empty key */
 
 	if (strcmp(key, a->key) != 0) {
-		/* key change: delete old attrib, create new attrib */
-		prio = a->prio;
-		val = a->val;
+		/* key change: rename attrib */
 		ctx->refresh_lock++;
-		uundo_freeze_serial(&ctx->sheet->undo);
-		csch_attr_modify_del(ctx->sheet, ctx->obj, a->key, 1);
-		csch_attr_modify_str(ctx->sheet, ctx->obj, prio, key, val, attrdlg_src(), 1);
-		uundo_unfreeze_serial(&ctx->sheet->undo);
-		uundo_inc_serial(&ctx->sheet->undo);
+		csch_attr_modify_rename(ctx->sheet, ctx->obj, a, key, attrdlg_src(), 1);
 		ctx->refresh_lock--;
 		sheet2dlg_cursor(ctx, key, 0);
 		return;
@@ -757,15 +752,13 @@ static const char *attr_dlg(csch_sheet_t *sheet, csch_cgrp_t *obj, const char *t
 
 	RND_DAD_BEGIN_VBOX(ctx->dlg);
 		RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-		RND_DAD_BEGIN_HPANE(ctx->dlg);
+		RND_DAD_BEGIN_HPANE(ctx->dlg, "left-right");
 			RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-			RND_DAD_SET_PANE_NAME(ctx->dlg, "left-right");
 
 			RND_DAD_BEGIN_VBOX(ctx->dlg); /* left */
 				RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
 				RND_DAD_LABEL(ctx->dlg, "Concrete:");
-				RND_DAD_BEGIN_VPANE(ctx->dlg);
-					RND_DAD_SET_PANE_NAME(ctx->dlg, "left_top-bottom");
+				RND_DAD_BEGIN_VPANE(ctx->dlg, "left_top-bottom");
 
 					RND_DAD_TREE(ctx->dlg, 3, 0, hdr);
 						RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL | RND_HATF_SCROLL);
@@ -875,7 +868,7 @@ static const char *attr_dlg(csch_sheet_t *sheet, csch_cgrp_t *obj, const char *t
 
 			RND_DAD_BEGIN_VBOX(ctx->dlg); /* top right */
 				RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-				aattr_dlg_create(&ctx->right, ctx->dlg, sheet->parent);
+				aattr_dlg_create(&ctx->right, ctx->dlg, (csch_project_t *)sheet->hidlib.project);
 			RND_DAD_END(ctx->dlg);
 		RND_DAD_END(ctx->dlg);
 
@@ -1000,7 +993,7 @@ const char csch_acts_AttributeDialog[] = "AttributeDialog([last-click|parent|obj
 const char csch_acth_AttributeDialog[] = "Bring up the Attribute Editor Dialog for an object.\n";
 fgw_error_t csch_act_AttributeDialog(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hidlib = RND_ACT_HIDLIB;
+	rnd_design_t *hidlib = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hidlib;
 	const char *cmd = "object", *target_key = NULL;
 	csch_chdr_t *obj = NULL;
@@ -1031,7 +1024,7 @@ const char csch_acts_AttributePick[] = "AttributePick([last-click|parent|object[
 const char csch_acth_AttributePick[] = "Bring up the Attribute Pick Dialog and ask the user to pick an attribute of an object.\n";
 fgw_error_t csch_act_AttributePick(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hidlib = RND_ACT_HIDLIB;
+	rnd_design_t *hidlib = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hidlib;
 	const char *cmd = "object", *target_key = NULL;
 	csch_chdr_t *obj = NULL;
@@ -1088,7 +1081,7 @@ void csch_dlg_attr_compiled(csch_project_t *prj)
 
 	for(n = gdl_first(&attrdlgs); n != NULL; n = next) {
 		next = gdl_next(&attrdlgs, n);
-		if (n->sheet->parent == prj)
+		if ((csch_project_t *)n->sheet->hidlib.project == prj)
 			sheet2dlg(n);
 	}
 }
