@@ -37,10 +37,10 @@
 #include <librnd/core/event.h>
 #include <librnd/core/actions.h>
 #include <librnd/core/hidlib.h>
-#include <librnd/core/hidlib_conf.h>
-#include <librnd/core/hid_dad.h>
-#include <librnd/core/hid_dad_tree.h>
-#include <librnd/core/tool.h>
+#include <librnd/core/rnd_conf.h>
+#include <librnd/hid/hid_dad.h>
+#include <librnd/hid/hid_dad_tree.h>
+#include <librnd/hid/tool.h>
 
 #include <libcschem/concrete.h>
 #include <libcschem/cnc_any_obj.h>
@@ -242,6 +242,8 @@ static const char *xpm_loc_edit[] = {
 
 /***/
 
+static void library_filter_reapply(void *hid_ctx, library_dlg_ctx_t *ctx);
+
 static long sheetlib_id(const csch_sheet_t *sheet, const csch_lib_master_t *master)
 {
 	if (master->uid > 255)
@@ -468,7 +470,7 @@ static rnd_bool library_mouse(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *pr
 	return rnd_false;
 }
 
-static void library_expose_grp(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *prv, rnd_hid_gc_t gc, const rnd_hid_expose_ctx_t *e)
+static void library_expose_grp(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *prv, rnd_hid_gc_t gc, rnd_hid_expose_ctx_t *e)
 {
 	library_dlg_ctx_t *ctx = prv->user_ctx;
 	rnd_xform_t xform = {0};
@@ -477,7 +479,7 @@ static void library_expose_grp(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *p
 	sch_rnd_draw_sheet(&ctx->prsh, gc, e, &xform);
 }
 
-static void library_expose(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *prv, rnd_hid_gc_t gc, const rnd_hid_expose_ctx_t *e)
+static void library_expose(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *prv, rnd_hid_gc_t gc, rnd_hid_expose_ctx_t *e)
 {
 	TODO("do this only if master has ");
 	library_expose_grp(attrib, prv, gc, e);
@@ -643,6 +645,7 @@ static void library_refresh_cb(void *hid_ctx, void *caller_data, rnd_hid_attribu
 	csch_lib_t *lroot;
 	rnd_hid_row_t *r = rnd_dad_tree_get_selected(attr);
 	const char *rootname;
+	char *freeme = NULL;
 
 	if (r == NULL) {
 		rnd_message(RND_MSG_ERROR, "Please select a subtree to refresh\n");
@@ -660,7 +663,7 @@ static void library_refresh_cb(void *hid_ctx, void *caller_data, rnd_hid_attribu
 	if ((rootname == NULL) || (*rootname == '\0'))
 		rootname = lroot->realpath;
 
-	rootname = rnd_strdup(rootname); /* rehash may change the ptr */
+	rootname = freeme = rnd_strdup(rootname); /* rehash may change the ptr */
 
 	if (csch_lib_rehash(ctx->sheet, ctx->master, r->user_data) == 0) {
 		rnd_message(RND_MSG_INFO, "Refresh library '%s'\n", rootname);
@@ -668,11 +671,12 @@ static void library_refresh_cb(void *hid_ctx, void *caller_data, rnd_hid_attribu
 
 		r = rnd_dad_tree_get_selected(attr);
 		library_update_preview(ctx, r->user_data, NULL);
+		library_filter_reapply(hid_ctx, ctx);
 	}
 	else
 		rnd_message(RND_MSG_ERROR, "Failed to refresh '%s'\n", rootname);
 
-	free(rootname);
+	free(freeme);
 }
 
 static rnd_hid_row_t *find_sym_prefix_(rnd_hid_tree_t *tree, gdl_list_t *rowlist, const char *name, int namelen)
@@ -758,9 +762,10 @@ static void library_edit_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_
 }
 
 
-static void library_filter_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr_inp)
+static void library_filter_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr_inp__)
 {
 	library_dlg_ctx_t *ctx = caller_data;
+	rnd_hid_attribute_t *attr_inp = &ctx->dlg[ctx->wfilt];
 	rnd_hid_attribute_t *attr;
 	rnd_hid_tree_t *tree;
 	const char *otext;
@@ -772,6 +777,7 @@ static void library_filter_cb(void *hid_ctx, void *caller_data, rnd_hid_attribut
 	attr = &ctx->dlg[ctx->wtree];
 	tree = attr->wdata;
 	otext = attr_inp->val.str;
+	if (otext == NULL) otext = "";
 	text = rnd_strdup(otext);
 	have_filter_text = (*text != '\0');
 
@@ -856,6 +862,12 @@ static void library_filter_cb(void *hid_ctx, void *caller_data, rnd_hid_attribut
 	free(text);
 }
 
+static void library_filter_reapply(void *hid_ctx, library_dlg_ctx_t *ctx)
+{
+	library_filter_cb(hid_ctx, ctx, NULL);
+}
+
+
 static csch_lib_t *loclib_get_lib(library_dlg_ctx_t *ctx)
 {
 	rnd_hid_attribute_t *attr = &ctx->dlg[ctx->wtree];
@@ -934,8 +946,7 @@ static char *sch_rnd_library_dlg(csch_sheet_t *sheet, const char *lib_type_name,
 
 	RND_DAD_BEGIN_VBOX(ctx->dlg);
 		RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-		RND_DAD_BEGIN_HPANE(ctx->dlg);
-			RND_DAD_SET_PANE_NAME(ctx->dlg, "left-right");
+		RND_DAD_BEGIN_HPANE(ctx->dlg, "left-right");
 			/* left */
 			RND_DAD_BEGIN_VBOX(ctx->dlg);
 				RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
@@ -982,9 +993,8 @@ static char *sch_rnd_library_dlg(csch_sheet_t *sheet, const char *lib_type_name,
 			RND_DAD_END(ctx->dlg);
 
 			/* right */
-			RND_DAD_BEGIN_VPANE(ctx->dlg);
+			RND_DAD_BEGIN_VPANE(ctx->dlg, "right_top-bottom");
 				RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL | RND_HATF_FRAME);
-				RND_DAD_SET_PANE_NAME(ctx->dlg, "right_top-bottom");
 
 				RND_DAD_BEGIN_VBOX(ctx->dlg);
 					/* right top: preview */
@@ -1022,7 +1032,7 @@ static char *sch_rnd_library_dlg(csch_sheet_t *sheet, const char *lib_type_name,
 	uundo_freeze_serial(&ctx->prsh.undo);
 	library_sheet2dlg(ctx);
 
-	ctx->title = rnd_strdup_printf("%s Library for %s", ctx->master->name, ctx->sheet->loadname);
+	ctx->title = rnd_strdup_printf("%s Library for %s", ctx->master->name, ctx->sheet->hidlib.loadname);
 	RND_DAD_NEW("library", ctx->dlg, ctx->title, ctx, modal, library_dlg_close_cb); /* type=local */
 
 	sheetlib_set(sheet, master, ctx);
@@ -1046,7 +1056,7 @@ const char csch_acts_LibraryDialog[] = "LibraryDialog([lib_type_name, [sheet|glo
 const char csch_acth_LibraryDialog[] = "Bring up the library dialog. Default lib_type_name is \"symbol\". \"Sheet\" is the sheet's local library (one dialog per sheet), \"global\" is all symbols mapped for any sheet seen in this session. In modal mode returns the tree path of the selected entry.\n";
 fgw_error_t csch_act_LibraryDialog(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hidlib = RND_ACT_HIDLIB;
+	rnd_design_t *hidlib = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hidlib;
 	int cmd = F_Global, modal = 0;
 	const char *lib_type_name = "symbol", *smodal = NULL;
