@@ -37,9 +37,9 @@
 #include <librnd/core/event.h>
 #include <librnd/core/actions.h>
 #include <librnd/core/hidlib.h>
-#include <librnd/core/hidlib_conf.h>
-#include <librnd/core/hid_dad.h>
-#include <librnd/core/hid_dad_tree.h>
+#include <librnd/core/rnd_conf.h>
+#include <librnd/hid/hid_dad.h>
+#include <librnd/hid/hid_dad_tree.h>
 
 #include <libcschem/concrete.h>
 #include <libcschem/project.h>
@@ -126,7 +126,7 @@ static const char *get_name(csch_chdr_t *obj)
 	csch_cgrp_t *grp;
 
 	if (obj == &obj->sheet->direct.hdr)
-		return obj->sheet->loadname;
+		return obj->sheet->hidlib.loadname;
 
 	if (!csch_obj_is_grp(obj))
 		return "";
@@ -193,8 +193,8 @@ static void tree_update_(tree_dlg_ctx_t *ctx, rnd_hid_attribute_t *attr, rnd_hid
 			cols[1] = rnd_strdup_printf("(%s)", csch_ctype_name(obj->type));
 		else
 			cols[1] = rnd_strdup(csch_ctype_name(obj->type));
-		cols[2] = rnd_strdup_null(subtype);
-		cols[3] = rnd_strdup_null(name);
+		cols[2] = rnd_strdup(subtype == NULL ? "" : subtype);
+		cols[3] = rnd_strdup(name == NULL ? "" : name);
 		cols[4] = NULL;
 	}
 
@@ -205,6 +205,7 @@ static void tree_update_(tree_dlg_ctx_t *ctx, rnd_hid_attribute_t *attr, rnd_hid
 		rnd_dad_tree_modify_cell(attr, row, 1, cols[1]);
 		rnd_dad_tree_modify_cell(attr, row, 2, cols[2]);
 		rnd_dad_tree_modify_cell(attr, row, 3, cols[3]);
+		free(cols[0]);
 	}
 	row->user_data = obj;
 	row->user_data2.lng++;
@@ -291,8 +292,8 @@ static void tree_update_details(tree_dlg_ctx_t *ctx, rnd_hid_row_t *row)
 static void tree_update_prj(tree_dlg_ctx_t *ctx, rnd_hid_attribute_t *attr)
 {
 	long n;
-	for(n = 0; n < ctx->prj->sheets.used; n++) {
-		csch_sheet_t *sheet = ctx->prj->sheets.array[n];
+	for(n = 0; n < ctx->prj->hdr.designs.used; n++) {
+		csch_sheet_t *sheet = ctx->prj->hdr.designs.array[n];
 		if (sheet != NULL)
 			tree_update_(ctx, attr, NULL, &sheet->direct.hdr);
 	}
@@ -439,7 +440,7 @@ static void tree_anyedit_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_
 	csch_chdr_t *obj = get_dlg_obj(ctx, 0);
 
 	if (obj != NULL) {
-		rnd_hidlib_t *hl = rnd_gui->get_dad_hidlib(hid_ctx);
+		rnd_design_t *hl = rnd_gui->get_dad_design(hid_ctx);
 		char *arg, *oidp;
 
 		oidp = csch_chdr_to_oidpath_str(obj);
@@ -462,13 +463,12 @@ static void tree_attredit_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute
 	tree_anyedit_cb(hid_ctx, caller_data, attr, "AttributeDialog");
 }
 
-static void tree_prv_expose_cb(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *prv, rnd_hid_gc_t gc, const rnd_hid_expose_ctx_t *e)
+static void tree_prv_expose_cb(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *prv, rnd_hid_gc_t gc, rnd_hid_expose_ctx_t *e)
 {
 	tree_dlg_ctx_t *ctx = prv->user_ctx;
 	rnd_xform_t xform = {0};
 	csch_chdr_t *obj = get_dlg_obj(ctx, 1);
 	int save, n;
-	rnd_hidlib_t *ssave;
 
 	/* draw the sheet */
 	if (obj != NULL) {
@@ -487,20 +487,13 @@ static void tree_prv_expose_cb(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *p
 		}
 	}
 
-	{
-	TODO("librnd: 4.0.0 can remove this workaround\n")
-	extern rnd_hidlib_t *csch_hidlib;
-	ssave = csch_hidlib;
 	if (obj != NULL)
-		csch_hidlib = &obj->sheet->hidlib;
+		e->design = &obj->sheet->hidlib;
 	else
-		csch_hidlib = rnd_gui->get_dad_hidlib(ctx->dlg_hid_ctx);
+		e->design = rnd_gui->get_dad_design(ctx->dlg_hid_ctx);
 
 	rnd_app.expose_main(rnd_gui, e, &xform);
 
-	csch_hidlib = ssave;
-	}
-
 	if (obj != NULL) {
 		obj->hilight = save;
 		if (obj->type == CSCH_CTYPE_CONN) {
@@ -606,9 +599,9 @@ static void sch_rnd_tree_dlg(csch_project_t *prj, csch_sheet_t *oidpath_sheet, c
 
 	RND_DAD_BEGIN_VBOX(ctx->dlg);
 		RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-		RND_DAD_BEGIN_HPANE(ctx->dlg);
+		RND_DAD_BEGIN_HPANE(ctx->dlg, "left-right");
 			RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-			RND_DAD_SET_PANE_NAME(ctx->dlg, "left-right");
+
 			RND_DAD_BEGIN_VBOX(ctx->dlg); /* left */
 				RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
 				RND_DAD_TREE(ctx->dlg, 4, 1, hdr); /* top left: tree */
@@ -633,9 +626,8 @@ static void sch_rnd_tree_dlg(csch_project_t *prj, csch_sheet_t *oidpath_sheet, c
 				ctx->wupdate = RND_DAD_CURRENT(ctx->dlg);
 			RND_DAD_END(ctx->dlg);
 
-			RND_DAD_BEGIN_VPANE(ctx->dlg); /* right */
+			RND_DAD_BEGIN_VPANE(ctx->dlg, "right_top/bottom"); /* right */
 				RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-				RND_DAD_SET_PANE_NAME(ctx->dlg, "right_top/bottom");
 
 				RND_DAD_PREVIEW(ctx->dlg, tree_prv_expose_cb, tree_prv_mouse_cb, NULL, NULL, &ctx->prvbb, 150, 150, ctx);
 					RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_PRV_GFLIP);
@@ -680,7 +672,7 @@ static void tree_timer_cb(rnd_hidval_t user_data)
 static void csch_dlg_tree_regen_(csch_sheet_t *sheet, int timeout)
 {
 	rnd_hidval_t user_data;
-	tree_dlg_ctx_t *ctx = htpp_get(&prj2dlg, sheet->parent);
+	tree_dlg_ctx_t *ctx = htpp_get(&prj2dlg, (csch_project_t *)sheet->hidlib.project);
 
 	if (ctx == NULL) return;
 
@@ -716,7 +708,7 @@ const char csch_acts_TreeDialog[] = "TreeDialog([object[=idpath]|objarr,vtp0ptr]
 const char csch_acth_TreeDialog[] = "Bring up the sheet's object tree dialog. If object is specified, move cursor to that object in the tree.\n";
 fgw_error_t csch_act_TreeDialog(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	rnd_hidlib_t *hidlib = RND_ACT_HIDLIB;
+	rnd_design_t *hidlib = RND_ACT_DESIGN;
 	csch_sheet_t *sheet = (csch_sheet_t *)hidlib;
 	int cmd = -1;
 	const char *cmds = "", *oidps;
@@ -740,7 +732,7 @@ fgw_error_t csch_act_TreeDialog(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 
 	switch(cmd) {
 		case -1:
-			sch_rnd_tree_dlg(sheet->parent, sheet, oidps, NULL);
+			sch_rnd_tree_dlg((csch_project_t *)sheet->hidlib.project, sheet, oidps, NULL);
 			break;
 		case F_Object:
 			{
@@ -753,7 +745,7 @@ fgw_error_t csch_act_TreeDialog(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 				obj = sch_rnd_search_obj_at(sheet, x, y, sch_rnd_slop);
 				if (obj != NULL) {
 					sheet = obj->sheet;
-					sch_rnd_tree_dlg(sheet->parent, sheet, csch_chdr_to_oidpath_str(obj), NULL);
+					sch_rnd_tree_dlg((csch_project_t *)sheet->hidlib.project, sheet, csch_chdr_to_oidpath_str(obj), NULL);
 				}
 				else
 					rnd_message(RND_MSG_ERROR, "TreeDialog(): no object under cursor\n");
@@ -766,7 +758,7 @@ fgw_error_t csch_act_TreeDialog(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 					rnd_message(RND_MSG_ERROR, "TreeDialog(): objarr argument is not an object arr pointer\n");
 					break;
 				}
-				sch_rnd_tree_dlg(sheet->parent, NULL, NULL, arr);
+				sch_rnd_tree_dlg((csch_project_t *)sheet->hidlib.project, NULL, NULL, arr);
 			}
 			break;
 		default:
