@@ -43,6 +43,7 @@
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/rnd_printf.h>
 #include <librnd/core/plugins.h>
+#include <librnd/core/conf_multi.h>
 
 #include <plugins/sch_dialogs/quick_attr_util.h>
 
@@ -199,8 +200,7 @@ int pplg_check_ver_std_cschem(int ver_needed) { return 0; }
 void pplg_uninit_std_cschem(void)
 {
 	rnd_remove_actions_by_cookie(std_cschem_cookie);
-	rnd_conf_unreg_intern(std_cschem_conf_internal);
-	rnd_conf_unreg_fields("plugins/std_cschem/");
+	rnd_conf_plug_unreg("plugins/std_cschem/", std_cschem_conf_internal, std_cschem_cookie);
 }
 
 int pplg_init_std_cschem(void)
@@ -211,7 +211,7 @@ int pplg_init_std_cschem(void)
 
 	RND_REGISTER_ACTIONS(std_cschem_action_list, std_cschem_cookie);
 
-	rnd_conf_reg_intern(std_cschem_conf_internal);
+	rnd_conf_plug_reg(std_cschem_conf, std_cschem_conf_internal, std_cschem_cookie);
 #define conf_reg(field,isarray,type_name,cpath,cname,desc,flags) \
 	rnd_conf_reg_field(std_cschem_conf, field,isarray,type_name,cpath,cname,desc,flags);
 #include "std_cschem_conf_fields.h"
