@@ -26,6 +26,8 @@
  *    mailing list: http://www.repo.hu/projects/sch-rnd/contact.html
  */
 
+#include <libcschem/cnc_obj.h>
+
 #include "tool_movecopy.h"
 
 typedef struct {
@@ -58,7 +60,7 @@ static void tool_movecopy_uninit(void)
 }
 
 /* for the passon from the arrow tool */
-static void tool_movecopy_press_at(rnd_hidlib_t *hl, csch_coord_t x, csch_coord_t y, csch_chdr_t *obj, int ret_arrow)
+static void tool_movecopy_press_at(rnd_design_t *hl, csch_coord_t x, csch_coord_t y, csch_chdr_t *obj, int ret_arrow)
 {
 	csch_sheet_t *sheet = (csch_sheet_t *)hl;
 	long v;
@@ -103,7 +105,7 @@ static void tool_movecopy_press_at(rnd_hidlib_t *hl, csch_coord_t x, csch_coord_
 	mvcp.click = 1;
 }
 
-static void tool_movecopy_press(rnd_hidlib_t *hl)
+static void tool_movecopy_press(rnd_design_t *hl)
 {
 	tool_movecopy_press_at(hl, sch_rnd_crosshair_x, sch_rnd_crosshair_y, NULL, 0);
 }
@@ -128,7 +130,7 @@ static void move_endpoint(csch_sheet_t *sheet, csch_coord_t dx, csch_coord_t dy)
 
 }
 
-static void tool_movecopy_release(rnd_hidlib_t *hl)
+static void tool_movecopy_release(rnd_design_t *hl)
 {
 	csch_sheet_t *sheet = (csch_sheet_t *)hl;
 	long n;
@@ -140,6 +142,7 @@ static void tool_movecopy_release(rnd_hidlib_t *hl)
 		rnd_trace("move release: %d %d\n", dx, dy);
 		if (!mvcp.copy) {
 			if (!mvcp.endpoint) {
+				csch_cobj_redraw_freeze(sheet);
 				uundo_freeze_serial(&sheet->undo);
 				csch_wirenet_recalc_freeze(sheet);
 				for(n = 0; n < mvcp.click_list.used; n++) {
@@ -152,11 +155,13 @@ static void tool_movecopy_release(rnd_hidlib_t *hl)
 				csch_wirenet_recalc_unfreeze(sheet);
 				uundo_unfreeze_serial(&sheet->undo);
 				uundo_inc_serial(&sheet->undo);
+				csch_cobj_redraw_unfreeze(sheet);
 			}
 			else
 				move_endpoint(sheet, dx, dy);
 		}
 		else {
+			csch_cobj_redraw_freeze(sheet);
 			uundo_freeze_serial(&sheet->undo);
 			csch_wirenet_recalc_freeze(sheet);
 			for(n = 0; n < mvcp.click_list.used; n++) {
@@ -169,6 +174,7 @@ static void tool_movecopy_release(rnd_hidlib_t *hl)
 			csch_wirenet_recalc_unfreeze(sheet);
 			uundo_unfreeze_serial(&sheet->undo);
 			uundo_inc_serial(&sheet->undo);
+			csch_cobj_redraw_unfreeze(sheet);
 		}
 
 
@@ -194,7 +200,7 @@ static void xor_draw_endpoint(csch_sheet_t *sheet, csch_coord_t dx, csch_coord_t
 	}
 }
 
-static void tool_movecopy_draw_attached(rnd_hidlib_t *hl)
+static void tool_movecopy_draw_attached(rnd_design_t *hl)
 {
 	if (mvcp.click) {
 		csch_sheet_t *sheet = (csch_sheet_t *)hl;
