@@ -123,9 +123,11 @@ int sch_rnd_buffer_copy_selected(csch_sheet_t *sheet, csch_sheet_t *buffer, csch
 		return -1;
 	}
 
-
+	csch_cobj_redraw_freeze(sheet);
 	uundo_freeze_serial(&sheet->undo);
 	uundo_freeze_add(&buffer->undo);
+	csch_wirenet_recalc_freeze(sheet);
+	csch_wirenet_recalc_freeze(buffer);
 
 	for(n = 0; n < objs.used; n++) {
 		csch_chdr_t *obj = objs.array[n], *newo;
@@ -168,10 +170,12 @@ int sch_rnd_buffer_copy_selected(csch_sheet_t *sheet, csch_sheet_t *buffer, csch
 			csch_op_remove(sheet, objs.array[n]);
 	}
 
+	csch_wirenet_recalc_unfreeze(buffer);
+	csch_wirenet_recalc_unfreeze(sheet);
 	uundo_unfreeze_add(&buffer->undo);
 	uundo_unfreeze_serial(&sheet->undo);
 	uundo_inc_serial(&sheet->undo);
-
+	csch_cobj_redraw_unfreeze(sheet);
 
 	vtp0_uninit(&objs);
 	return 0;
@@ -182,6 +186,7 @@ void sch_rnd_buffer_paste(csch_sheet_t *sheet, csch_sheet_t *buffer, csch_coord_
 	int warned = 0;
 	htip_entry_t *e;
 
+	csch_cobj_redraw_freeze(sheet);
 	uundo_freeze_serial(&sheet->undo);
 	csch_wirenet_recalc_freeze(sheet);
 
@@ -245,6 +250,7 @@ void sch_rnd_buffer_paste(csch_sheet_t *sheet, csch_sheet_t *buffer, csch_coord_
 	csch_wirenet_recalc_unfreeze(sheet);
 	uundo_unfreeze_serial(&sheet->undo);
 	uundo_inc_serial(&sheet->undo);
+	csch_cobj_redraw_unfreeze(sheet);
 }
 
 int sch_rnd_buffer_clear(csch_sheet_t *buffer)
@@ -266,7 +272,7 @@ static const char csch_acts_BufferPaste[] = "BufferPaste([x, y])";
 static const char csch_acth_BufferPaste[] = "Paste selection from active paste buffer to the board at crosshair or x;y.\n";
 fgw_error_t csch_act_BufferPaste(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	csch_sheet_t *sheet = (csch_sheet_t *)RND_ACT_HIDLIB;
+	csch_sheet_t *sheet = (csch_sheet_t *)RND_ACT_DESIGN;
 	csch_coord_t x = P2C(sch_rnd_crosshair_x), y = P2C(sch_rnd_crosshair_y);
 
 	RND_ACT_MAY_CONVARG(1, FGW_COORD, BufferPaste, x = fgw_coord(&(argv[1])));
@@ -281,7 +287,7 @@ static const char csch_acts_BufferCopy[] = "BufferCopy([x, y])";
 static const char csch_acth_BufferCopy[] = "Copy selection to currently active paste buffer.\n";
 fgw_error_t csch_act_BufferCopy(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	csch_sheet_t *sheet = (csch_sheet_t *)RND_ACT_HIDLIB;
+	csch_sheet_t *sheet = (csch_sheet_t *)RND_ACT_DESIGN;
 	csch_coord_t x = P2C(sch_rnd_crosshair_x), y = P2C(sch_rnd_crosshair_y);
 
 	RND_ACT_MAY_CONVARG(1, FGW_COORD, BufferCopy, x = fgw_coord(&(argv[1])));
@@ -295,7 +301,7 @@ static const char csch_acts_BufferCut[] = "BufferCut()";
 static const char csch_acth_BufferCut[] = "Cut selection to currently active paste buffer: copy objects then remove them from the sheet.\n";
 fgw_error_t csch_act_BufferCut(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	csch_sheet_t *sheet = (csch_sheet_t *)RND_ACT_HIDLIB;
+	csch_sheet_t *sheet = (csch_sheet_t *)RND_ACT_DESIGN;
 	RND_ACT_IRES(sch_rnd_buffer_copy_selected(sheet, SCH_RND_PASTEBUFFER, P2C(sch_rnd_crosshair_x), P2C(sch_rnd_crosshair_y), 1));
 	return 0;
 }
@@ -390,7 +396,7 @@ fgw_error_t csch_act_BufferLoad(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		static char *default_file = NULL;
 		fn = rnd_hid_fileselect(rnd_gui, "Load group into buffer...",
 			"Choose a file to load the group from\ninto the paste buffer.\n",
-			default_file, ".*", NULL, "grp", RND_HID_FSD_READ | RND_HID_FSD_MAY_NOT_EXIST, NULL);
+			default_file, ".*", NULL, "grp", RND_HID_FSD_READ, NULL);
 		if (default_file != NULL) {
 			free(default_file);
 			default_file = NULL;
@@ -448,7 +454,7 @@ static const char csch_acts_ReplaceSymbol[] = "ReplaceSymbol([object])";
 static const char csch_acth_ReplaceSymbol[] = "Replace target symbol with the first symbol from the buffer\n";
 fgw_error_t csch_act_ReplaceSymbol(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 {
-	csch_sheet_t *sheet = (csch_sheet_t *)RND_ACT_HIDLIB;
+	csch_sheet_t *sheet = (csch_sheet_t *)RND_ACT_DESIGN;
 	int op = F_Object;
 	csch_chdr_t *dst;
 	csch_cgrp_t *dst_grp, *src_grp;
