@@ -26,8 +26,8 @@
 
 #include <libcschem/config.h>
 
-#include <librnd/core/hid.h>
-#include <librnd/core/hid_inlines.h>
+#include <librnd/hid/hid.h>
+#include <librnd/hid/hid_inlines.h>
 #include <librnd/core/color.h>
 #include <librnd/core/vtc0.h>
 #include <librnd/core/compat_misc.h>
@@ -50,9 +50,6 @@
 #include "draw.h"
 #include "search.h"
 
-TODO("REMOVE")
-extern rnd_hidlib_t *csch_hidlib;
-
 static const char draw_cookie[] = "sch-rnd/draw.c";
 
 typedef struct {
@@ -551,7 +548,7 @@ void sch_rnd_expose_main(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *region, rnd
 {
 	rnd_hid_gc_t gc;
 	int want_layer;
-	csch_sheet_t *sheet = (csch_sheet_t *)csch_hidlib;
+	csch_sheet_t *sheet = (csch_sheet_t *)region->design;
 	rnd_hid_t *save = rnd_render;
 	
 	if ((rnd_render != NULL) && (!rnd_render->override_render))
@@ -563,7 +560,7 @@ void sch_rnd_expose_main(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *region, rnd
 
 	/* the HID needs a group set, even if we don't do groups */
 	if (rnd_render->set_layer_group != NULL)
-		want_layer = rnd_render->set_layer_group(rnd_render, 1, NULL, 0, 0, 0, 0, NULL);
+		want_layer = rnd_render->set_layer_group(rnd_render, region->design, 1, NULL, 0, 0, 0, 0, NULL);
 	else
 		want_layer = 0;
 
@@ -583,7 +580,7 @@ void sch_rnd_expose_main(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *region, rnd
 }
 
 
-void sch_rnd_expose_preview(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *e)
+void sch_rnd_expose_preview(rnd_hid_t *hid, rnd_hid_expose_ctx_t *e)
 {
 	rnd_hid_gc_t gc = rnd_render->make_gc(rnd_render);
 
@@ -596,24 +593,33 @@ void sch_rnd_expose_preview(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *e)
 	rnd_render->destroy_gc(gc);
 }
 
-void sch_rnd_redraw(rnd_hidlib_t *target)
+void sch_rnd_redraw(rnd_design_t *target)
 {
-	if ((target == NULL) || (target == csch_hidlib))
+	rnd_design_t *curr = rnd_multi_get_current();
+
+	if ((target == NULL) || (target == curr))
 		rnd_render->invalidate_all(rnd_render);
 }
 
-
-void sch_rnd_obj_redraw_ev(rnd_hidlib_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[])
+void sch_rnd_box_redraw_ev(rnd_design_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[])
 {
-	csch_chdr_t *obj = argv[1].d.p;
+	csch_rtree_box_t *box = argv[1].d.p;
+	rnd_design_t *curr = rnd_multi_get_current();
+
+	if (curr != hidlib)
+		return;
+
 	assert(argv[1].type == RND_EVARG_PTR);
-	if (rnd_render->gui && (rnd_render->invalidate_lr != NULL) && (obj->bbox.x2 > obj->bbox.x1))
-		rnd_render->invalidate_lr(rnd_render, C2P(obj->bbox.x1), C2P(obj->bbox.x2), C2P(obj->bbox.y1), C2P(obj->bbox.y2));
+	if (rnd_render->gui && (rnd_render->invalidate_lr != NULL))
+		rnd_render->invalidate_lr(rnd_render, C2P(box->x1), C2P(box->x2), C2P(box->y1), C2P(box->y2));
+
+	/* rnd_trace("LR: %rc %rc - %rc %rc\n", box->x1, box->y1, box->x2, box->y2); */
+
 }
 
 void sch_rnd_draw_init2(void)
 {
-	rnd_event_bind(CSCH_EVENT_OBJ_NEEDS_REDRAW, sch_rnd_obj_redraw_ev, NULL, draw_cookie);
+	rnd_event_bind(CSCH_EVENT_BOX_NEEDS_REDRAW, sch_rnd_box_redraw_ev, NULL, draw_cookie);
 }
 
 void sch_rnd_draw_uninit(void)
