@@ -4,8 +4,8 @@
 #include <libcschem/concrete.h>
 
 /* coordinate <-> "pixel" conversion */
-#define C2P(coord) ((coord) << 10)
-#define P2C(coord) ((coord) >> 10)
+#define C2P(coord) ((rnd_coord_t)((coord) << 10))
+#define P2C(coord) ((csch_coord_t)((coord) >> 10))
 
 struct rnd_xform_s {
 	unsigned faded:1;
@@ -17,10 +17,10 @@ struct rnd_xform_s {
 };
 
 /* redraw the entire screen */
-void sch_rnd_redraw(rnd_hidlib_t *target);
+void sch_rnd_redraw(rnd_design_t *target);
 
 void sch_rnd_expose_main(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *region, rnd_xform_t *xform_caller);
-void sch_rnd_expose_preview(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *e);
+void sch_rnd_expose_preview(rnd_hid_t *hid, rnd_hid_expose_ctx_t *e);
 
 
 /*** calls useful in preview ***/
