@@ -31,8 +31,8 @@
 #include <ctype.h>
 #include <genht/htsp.h>
 #include <genht/hash.h>
-#include <rnd_inclib/font/font.h>
-#include <librnd/core/hid_inlines.h>
+#include <librnd/font/font.h>
+#include <librnd/hid/hid_inlines.h>
 #include <librnd/core/rotate.h>
 #include <librnd/core/safe_fs_dir.h>
 
@@ -111,9 +111,9 @@ static lht_node_t *hash_get(lht_node_t *hash, const char *name)
 #define PARSE_COORD(dst, src)       parse_coord(dst, src)
 #define PARSE_DOUBLE(dst, src)      parse_double(dst, src)
 #define HASH_GET(hash, name)        hash_get(hash, name)
-#define LHT_ERROR                   iolht_error
+#define RND_LHT_ERROR               iolht_error
 
-#include <rnd_inclib/font/font_lht.h>
+#include <librnd/font/font_lht.h>
 
 
 /* text object fields:
@@ -121,7 +121,7 @@ static lht_node_t *hash_get(lht_node_t *hash, const char *name)
    - net_sx: unrotated text's net width, <= crd_sx
 */
 
-static rnd_font_t *rf_font_lookup_(rnd_hidlib_t *hidlib, const char *name, const char *style)
+static rnd_font_t *rf_font_lookup_(rnd_design_t *hidlib, const char *name, const char *style)
 {
 	long best_score;
 	htsp_entry_t *e, *best;
@@ -206,7 +206,7 @@ static rnd_font_t *rf_font_lookup_(rnd_hidlib_t *hidlib, const char *name, const
 	return best == NULL ? NULL : best->value;
 }
 
-static rnd_font_t *rf_font_lookup(rnd_hidlib_t *hidlib, const char *name, const char *style)
+static rnd_font_t *rf_font_lookup(rnd_design_t *hidlib, const char *name, const char *style)
 {
 	rnd_font_t *res = rf_font_lookup_(hidlib, name, style);
 
@@ -225,7 +225,7 @@ static rnd_font_t *rf_font_lookup(rnd_hidlib_t *hidlib, const char *name, const
 	return res;
 }
 
-static rnd_font_t *rf_resolve_pen(rnd_hidlib_t *hidlib, csch_cpen_t *pen)
+static rnd_font_t *rf_resolve_pen(rnd_design_t *hidlib, csch_cpen_t *pen)
 {
 	static int epic_fail = 0;
 	rnd_font_t *f;
@@ -309,6 +309,7 @@ RND_INLINE csch_coord_t rf_text_halign(const unsigned char *str, csch_halign_t h
 	double left;
 
 	switch(halign) {
+		case CSCH_HALIGN_invalid:
 		case CSCH_HALIGN_START:           *net_sx = used; return 0;
 		case CSCH_HALIGN_CENTER: center:; *net_sx = used; return rnd_round((double)(avail - used) / 2.0);
 		case CSCH_HALIGN_END:             *net_sx = used; return avail - used;
@@ -479,7 +480,7 @@ static void rf_inval_font(csch_sheet_t *sheet, csch_text_t *text)
 	text->scale = 0;
 }
 
-static void map_fontdir(rnd_hidlib_t *hidlib, const char *dr, int depth)
+static void map_fontdir(rnd_design_t *hidlib, const char *dr, int depth)
 {
 	DIR *dir;
 	struct dirent *de;
