@@ -36,11 +36,11 @@
 
 /* hidlib headers */
 #include <librnd/core/unit.h>
-#include <librnd/core/hid_init.h>
-#include <librnd/core/hid.h>
+#include <librnd/hid/hid_init.h>
+#include <librnd/hid/hid.h>
 #include <librnd/core/conf.h>
 #include <librnd/core/hidlib.h>
-#include <librnd/core/buildin.hidlib.h>
+#include <librnd/hid/buildin.hidlib.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/plugins.h>
 
@@ -54,8 +54,9 @@
 #include <libcschem/project.h>
 #include <libcschem/abstract.h>
 #include <libcschem/compile.h>
-#include <libcschem/export.h>
+#include <libcschem/util_export.h>
 #include <libcschem/event.h>
+#include <libcschem/csch_printf.h>
 
 #include "conf_core.h"
 #include "crosshair.h"
@@ -66,7 +67,8 @@
 #include "emergency.h"
 #include "project.h"
 #include "multi.h"
-
+#include "plug_io_act.h"
+#include "export.h"
 
 csch_chdr_t *csch_obj_clicked = NULL;
 
@@ -74,13 +76,7 @@ csch_chdr_t *csch_obj_clicked = NULL;
 #include "buildin.c"
 #undef pup_buildins
 
-
-TODO("multi: REMOVE")
-rnd_hidlib_t *csch_hidlib;
-TODO("multi: REMOVE")
-csch_project_t *prj; /* the project we are working on */
-
-static const char *EXPERIMENTAL = "The code is in beta testing state.";
+static const char *EXPERIMENTAL = NULL;
 
 static const char *menu_file_paths[8];
 static const char *menu_name_fmt = "menu-%s.lht";
@@ -113,10 +109,10 @@ static char *main_path_init(char *exec_prefix)
 	menu_file_paths[3] = NULL;
 	assert((sizeof(menu_file_paths)/sizeof(menu_file_paths[0])) > 3);
 
-	rnd_pcbhl_conf_postproc(); /* get ~ (home dir) set */
+	rnd_conf_postproc(); /* get ~ (home dir) set */
 
 	rnd_app.conf_sysdir_path = SCHSHAREDIR;
-	rnd_app.conf_sys_path = SCHSHAREDIR "/sch-rnd-conf.lht";
+	rnd_app.conf_sys_path = SCHCONFDIR "/sch-rnd-conf.lht";
 	rnd_app.lib_dir = SCHLIBDIR;
 	rnd_app.conf_userdir_path = CONF_USER_DIR;
 	rnd_app.conf_user_path = rnd_concat(CONF_USER_DIR, "/conf_core.lht", NULL);
@@ -260,10 +256,9 @@ extern void sch_rnd_project_uninit(void);
 
 void sch_rnd_main_uninit(void)
 {
-	TODO("multi: instead of this, free all sheets and projects using multi.h");
-	csch_hidlib = NULL;
 	sch_rnd_multi_uninit();
 	rnd_funchash_uninit();
+	csch_plug_io_act_uninit();
 	csch_uninit();
 	sch_rnd_draw_uninit();
 	sch_rnd_font_uninit();
@@ -274,6 +269,7 @@ void sch_rnd_main_uninit(void)
 	rnd_hidlib_uninit(); /* plugin unload */
 	main_path_uninit();
 	fgw_atexit();
+	conf_core_uninit();
 }
 
 extern const char *rnd_conf_internal;
@@ -286,6 +282,7 @@ int main(int argc, char *argv[])
 	char *exec_prefix;
 	rnd_main_args_t ga;
 	csch_sheet_t *first_sheet = NULL;
+	sch_rnd_export_appspec_t appspec = {0};
 
 	rnd_app.package = "sch-rnd";
 	rnd_app.version = SCH_VERSION;
@@ -308,9 +305,7 @@ int main(int argc, char *argv[])
 	rnd_app.conf_dont_merge_node = sch_rnd_conf_dont_merge_node;
 	rnd_app.multi_design = 1;
 
-	/* temporary hack for librnd 3.2.x; should be removed when this becomes
-	   the default in librnd 4 */
-	rnd_hid_enable_per_dialog_hidlib = 1;
+	rnd_printf_app_format = csch_printf_app_format;
 
 	rnd_fix_locale_and_env();
 	exec_prefix = rnd_exec_prefix(argv[0], BINDIR, BINDIR_TO_EXECPREFIX);
@@ -318,8 +313,7 @@ int main(int argc, char *argv[])
 	sch_rnd_csch_init();
 
 	rnd_main_args_init(&ga, argc, sch_rnd_action_args);
-	rnd_hidlib_init1(conf_core_init);
-	rnd_conf_set(RND_CFR_INTERNAL, "rc/path/exec_prefix", -1, exec_prefix, RND_POL_OVERWRITE);
+	rnd_hidlib_init1(conf_core_init, exec_prefix);
 	free(exec_prefix);
 
 	csch_event_init_app();
@@ -328,7 +322,9 @@ int main(int argc, char *argv[])
 
 	rnd_hidlib_init2(pup_buildins, sch_rnd_buildins);
 
+	csch_init_units();
 	gui_act_init();
+	csch_plug_io_act_init();
 	sch_rnd_main_init();
 	rnd_conf_set(RND_CFR_CLI, "editor/view/flip_y", 0, "1", RND_POL_OVERWRITE);
 
@@ -362,44 +358,51 @@ TODO("action list");
 	for(n = 0; ga.hid_argc > 0; n++, ga.hid_argc--) {
 		const char *fn = ga.hid_argv[n];
 		csch_sheet_t *sheet;
+		int is_project;
 
-		if (first_fn == NULL)
+		sheet = sch_rnd_multi_load(fn, fmt, &is_project);
+		if ((first_fn == NULL) && !is_project)
 			first_fn = fn;
-		sheet = sch_rnd_multi_load(fn, fmt);
-		if ((sheet == NULL) && rnd_main_exporting) {
-			rnd_message(RND_MSG_ERROR, "Can not load file '%s' (specified on command line) for exporting or printing\n", fn);
-			rnd_log_print_uninit_errs("Export load error");
-			exit(1);
+
+		/* check if failed to load the sheet */
+		if (sheet == NULL) {
+			if (rnd_main_exporting) {
+				rnd_message(RND_MSG_ERROR, "Can not load file '%s' (specified on command line) for exporting or printing\n", fn);
+				rnd_log_print_uninit_errs("Export load error");
+				exit(1);
+			}
+			else if (!sch_rnd_is_fn_project_file(fn)) {
+				sheet = sch_rnd_multi_new_empty(fn);
+				rnd_message(RND_MSG_INFO, "Creating new empty sheet for non-existing file '%s' (specified on command line)\n", fn);
+				if (sheet == NULL) {
+					rnd_log_print_uninit_errs("Sheet creation error");
+					fprintf(stderr, "Failed to create new sheet (by loading the default sheet), exiting\n");
+					sch_rnd_main_uninit();
+					rnd_main_args_uninit(&ga);
+					exit(1);
+				}
+			}
+			else {
+				rnd_log_print_uninit_errs("Project load error");
+				fprintf(stderr, "Failed to load project '%s', exiting\n", fn);
+				sch_rnd_main_uninit();
+				rnd_main_args_uninit(&ga);
+				exit(1);
+			}
 		}
+
+
 		if (first_sheet == NULL)
 			first_sheet = sheet;
-	}
-
-	if (first_sheet == NULL) {
-		first_sheet = sch_rnd_multi_new_empty();
-		if (first_sheet == NULL) {
-			fprintf(stderr, "Failed to create new sheet (by loading the default sheet)\n");
-			sch_rnd_main_uninit();
-			rnd_main_args_uninit(&ga);
-			exit(1);
-		}
-
-		/* If we failed to load a file with a given file name and created an
-		   empty sheet instead, remember the file name for the new sheet so
-		   {f s} saves using that name specified originally */
-		if (first_fn != NULL) {
-			first_sheet->loadname = rnd_strdup(first_fn);
-			first_sheet->hidlib.filename = rnd_strdup(first_fn);
-		}
+		else
+			appspec.exp_prj = 1;
+		if (is_project)
+			appspec.exp_prj = 1;
 	}
 
 	main_sighand_init();
 
-	TODO("multi: remove this at librnd4.0.0");
-	csch_hidlib = &first_sheet->hidlib; /* needed for exporters until librnd API passes this on */
-	
-	if (rnd_main_exported(&ga, &first_sheet->hidlib, 0)) {
-		TODO("multi: remove this when switching to librnd4.0.0; this is needed here now to the hackish config save/load uninit to work at exit");
+	if (rnd_main_exported(&ga, &first_sheet->hidlib, 0, &appspec)) {
 		sch_rnd_multi_switch_to_initial(first_sheet);
 
 		sch_rnd_main_uninit();
@@ -407,12 +410,19 @@ TODO("action list");
 		exit(0);
 	}
 
-	TODO("multi: remove this at librnd4.0.0");
-	csch_hidlib = NULL; /* have to set back for proper switch */
+	/* started with no sheet, create an unknown */
+	if (first_sheet == NULL) {
+		first_sheet = sch_rnd_multi_new_empty(NULL);
+		if (first_sheet == NULL) {
+			fprintf(stderr, "Failed to create new sheet (by loading the default sheet)\n");
+			sch_rnd_main_uninit();
+			rnd_main_args_uninit(&ga);
+			exit(1);
+		}
+	}
 
-
-	sch_rnd_crosshair_gui_init();
 	sch_rnd_multi_switch_to_initial(first_sheet);
+	sch_rnd_crosshair_gui_init();
 
 
 	sch_rnd_enable_autosave();
