@@ -2,7 +2,8 @@
 #define SCH_RND_SEARCH_H
 
 #include <libcschem/config.h>
-#include <librnd/core/grid.h>
+#include <librnd/hid/hid.h>
+#include <librnd/hid/grid.h>
 #include <libcschem/concrete.h>
 
 
