@@ -40,9 +40,10 @@
 #include <libcschem/plug_io.h>
 
 #include <librnd/core/conf.h>
-#include <librnd/core/hid_menu.h>
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/hid/hid_menu.h>
+#include <librnd/core/rnd_conf.h>
 #include <librnd/core/compat_fs.h>
+#include <librnd/core/compat_misc.h>
 #include <librnd/core/safe_fs.h>
 #include <librnd/core/paths.h>
 
@@ -69,6 +70,30 @@ csch_inline rnd_coord_t sheet_attr_crd(csch_sheet_t *sheet, const char *key, rnd
 	return val;
 }
 
+rnd_coord_t sch_rnd_sheet_attr_crd(csch_sheet_t *sheet, const char *key, rnd_coord_t defval)
+{
+	return sheet_attr_crd(sheet, key, defval);
+}
+
+static void warn_size(csch_sheet_t *sheet, csch_coord_t c)
+{
+	if (sheet->warned_too_large)
+		return;
+
+	if (c > P2C(RND_COORD_MAX)/2) {
+		rnd_message(RND_MSG_ERROR, "Sheet is too large. Please consider recompiling librnd with 64 bit coords.\n");
+		sheet->warned_too_large = 1;
+	}
+}
+
+void sch_rnd_sheet_warn_size(csch_sheet_t *sheet, csch_coord_t x1, csch_coord_t y1, csch_coord_t x2, csch_coord_t y2)
+{
+	if (sheet->warned_too_large)
+		return;
+	warn_size(sheet, x1); warn_size(sheet, y1);
+	warn_size(sheet, x2); warn_size(sheet, y2);
+}
+
 void sch_rnd_sheet_postproc(csch_sheet_t *sheet)
 {
 	rnd_coord_t min_x = sheet_attr_crd(sheet, "drawing_min_width", 0);
@@ -77,14 +102,16 @@ void sch_rnd_sheet_postproc(csch_sheet_t *sheet)
 	sheet->hidlib.grid = rnd_conf.editor.grid;
 	csch_sheet_bbox_update(sheet);
 
-	sheet->hidlib.size_x = C2P(RND_MAX(min_x, sheet->bbox.x2));
-	sheet->hidlib.size_y = C2P(RND_MAX(min_y, sheet->bbox.y2));
+	sheet->hidlib.dwg.X1 = C2P(sheet->bbox.x1);
+	sheet->hidlib.dwg.Y1 = C2P(sheet->bbox.y1);
+	sheet->hidlib.dwg.X2 = C2P(RND_MAX(min_x, sheet->bbox.x2));
+	sheet->hidlib.dwg.Y2 = C2P(RND_MAX(min_y, sheet->bbox.y2));
 
-	TODO("multi: implement sheet stack; temporary make sheet active for the config, then pop at end\n");
-	csch_lib_add_all(sheet, csch_lib_get_master("symbol", 1), &conf_core.rc.library_search_paths);
+	sch_rnd_sheet_warn_size(sheet, sheet->bbox.x1, sheet->bbox.y1, RND_MAX(min_x, sheet->bbox.x2), RND_MAX(min_y, sheet->bbox.y2));
 
-	if (sheet->hidlib.filename != NULL) {
-		sheet->design_dir = rnd_dirname(sheet->hidlib.filename);
+
+	if (sheet->hidlib.fullpath != NULL) {
+		sheet->design_dir = rnd_dirname(sheet->hidlib.fullpath);
 		rnd_conf_force_set_str(conf_core.rc.path.design, sheet->design_dir);
 	}
 	else {
@@ -92,16 +119,20 @@ void sch_rnd_sheet_postproc(csch_sheet_t *sheet)
 		rnd_conf_force_set_str(conf_core.rc.path.design, "");
 	}
 	rnd_conf_ro("rc/path/design");
+
+	/* This works from load only because the conf state is already set up for
+	   the new sheet, even tho librnd's current sheet is not yet set */
+	csch_lib_add_all(sheet, csch_lib_get_master("symbol", 1), &conf_core.rc.library_search_paths);
 }
 
-static csch_sheet_t *sch_rnd_sheet_new_(csch_project_t *prj, csch_sheet_t *dst_sheet)
+static csch_sheet_t *sch_rnd_sheet_new_(csch_project_t *prj, csch_sheet_t *dst_sheet, int emit_ev)
 {
 	csch_sheet_t *sheet = NULL;
 	int dsch = -1;
 
 	rnd_hid_menu_merge_inhibit_inc();
 	if (dst_sheet == NULL)
-		rnd_conf_list_foreach_path_first(NULL, dsch, &conf_core.rc.default_sheet_file, csch_project_load_sheet(prj, __path__, NULL, &sheet));
+		rnd_conf_list_foreach_path_first(NULL, dsch, &conf_core.rc.default_sheet_file, csch_project_load_sheet(prj, __path__, NULL, &sheet, 1));
 	else
 		rnd_conf_list_foreach_path_first(NULL, dsch, &conf_core.rc.default_sheet_file, ((sheet = csch_load_sheet_io(NULL, dst_sheet, __path__, __path__, NULL, 0)) == NULL));
 	rnd_hid_menu_merge_inhibit_dec();
@@ -124,7 +155,7 @@ static csch_sheet_t *sch_rnd_sheet_new_(csch_project_t *prj, csch_sheet_t *dst_s
 					dsch = 1;
 			}
 			else
-				dsch = csch_project_load_sheet(prj, efn, NULL, &sheet);
+				dsch = csch_project_load_sheet(prj, efn, NULL, &sheet, 0);
 
 			if (dsch == 0)
 				rnd_message(RND_MSG_WARNING, "Couldn't find default sheet - using the embedded fallback\n");
@@ -140,46 +171,42 @@ static csch_sheet_t *sch_rnd_sheet_new_(csch_project_t *prj, csch_sheet_t *dst_s
 		if (dst_sheet != NULL)
 			sheet = dst_sheet;
 		else
-			sheet = csch_sheet_alloc(dst_sheet);
+			sheet = csch_sheet_alloc(prj);
 	}
 
 	/* do not remember default sheet's file name so that a 'save' won't overwrite it */
-	free(sheet->hidlib.filename);
-	sheet->hidlib.filename = NULL;
-	sheet->newname = sheet->loadname;
-	sheet->loadname = NULL;
+	free(sheet->hidlib.fullpath);
+	sheet->hidlib.fullpath = NULL;
+	sheet->newname = sheet->hidlib.loadname;
+	sheet->hidlib.loadname = NULL;
 	minuid_gen(&csch_minuid, sheet->direct.uuid);
 	minuid_clr(sheet->direct.data.grp.src_uuid);
 
-	rnd_event(&sheet->hidlib, CSCH_EVENT_SHEET_POSTLOAD, NULL);
+	if (emit_ev)
+		rnd_event(&sheet->hidlib, CSCH_EVENT_SHEET_POSTLOAD, NULL);
 
 	return sheet;
 }
 
 csch_sheet_t *sch_rnd_sheet_new(csch_project_t *prj, csch_sheet_t *sheet)
 {
-	return sch_rnd_sheet_new_(prj, NULL);
+	return sch_rnd_sheet_new_(prj, NULL, 0);
 }
 
 csch_sheet_t *sch_rnd_sheet_new4revert(csch_sheet_t *sheet)
 {
-	return sch_rnd_sheet_new_(NULL, sheet);
+	csch_sheet_t *newsh = sch_rnd_sheet_new_(NULL, sheet, 1);
+	if (newsh->hidlib.fullpath == NULL)
+		newsh->hidlib.fullpath = rnd_strdup("<none>");
+	return newsh;
 }
 
 
-static void sch_rnd_sheet_postload_ev(rnd_hidlib_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[])
+static void sch_rnd_sheet_postload_ev(rnd_design_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[])
 {
 	csch_sheet_t *sheet = (csch_sheet_t *)hidlib;
 
 	sch_rnd_sheet_postproc(sheet);
-
-	TODO("multi: remove this");
-/* We used to do this here after load to get the loaded sheet active,
-   but multi handles this well now */
-/*	csch_hidlib = &sheet->hidlib;
-	if (rnd_gui->set_hidlib != NULL)
-		rnd_gui->set_hidlib(rnd_gui, hidlib);
-	rnd_event(hidlib, RND_EVENT_BOARD_FN_CHANGED, NULL);*/
 }
 
 void sch_rnd_sheet_init2(void)
